@@ -1,17 +1,18 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
-"""In this tutorial, we will train an image classifier with FLSim to simulate federated learning training environment.
+"""In this tutorial, we will train an image classifier with FLSim to simulate a federated learning training environment.
 
 With this tutorial, you will learn the following key components of FLSim:
 1. Data loading
 2. Model construction
 3. Trainer construction
 
-  Typical usage example:
+    Typical usage example:
     python3 cifar10_example.py --config-file configs/cifar10_config.json
 """
 import flsim.configs  # noqa
@@ -23,9 +24,9 @@ from flsim.utils.config_utils import maybe_parse_json_config
 from flsim.utils.example_utils import (
     DataLoader,
     DataProvider,
-    SimpleConvNet,
     FLModel,
     MetricsReporter,
+    SimpleConvNet,
 )
 from hydra.utils import instantiate
 from omegaconf import DictConfig, OmegaConf
@@ -36,7 +37,7 @@ from torchvision.datasets.cifar import CIFAR10
 IMAGE_SIZE = 32
 
 
-def build_data_provider(local_batch_size, examples_per_user, drop_last=False):
+def build_data_provider(local_batch_size, examples_per_user, drop_last: bool = False):
 
     transform = transforms.Compose(
         [
@@ -57,18 +58,19 @@ def build_data_provider(local_batch_size, examples_per_user, drop_last=False):
         train_dataset, test_dataset, test_dataset, sharder, local_batch_size, drop_last
     )
     data_provider = DataProvider(fl_data_loader)
-    print(f"Clients in total: {data_provider.num_users()}")
+    print(f"Clients in total: {data_provider.num_train_users()}")
     return data_provider
 
 
 def main(
     trainer_config,
     data_config,
-    use_cuda_if_available=True,
-):
+    use_cuda_if_available: bool = True,
+) -> None:
     cuda_enabled = torch.cuda.is_available() and use_cuda_if_available
     device = torch.device(f"cuda:{0}" if cuda_enabled else "cpu")
     model = SimpleConvNet(in_channels=3, num_classes=10)
+    # pyre-fixme[6]: Expected `Optional[str]` for 2nd param but got `device`.
     global_model = FLModel(model, device)
     if cuda_enabled:
         global_model.fl_cuda()
@@ -84,19 +86,19 @@ def main(
 
     final_model, eval_score = trainer.train(
         data_provider=data_provider,
-        metric_reporter=metrics_reporter,
-        num_total_users=data_provider.num_users(),
+        metrics_reporter=metrics_reporter,
+        num_total_users=data_provider.num_train_users(),
         distributed_world_size=1,
     )
 
     trainer.test(
-        data_iter=data_provider.test_data(),
-        metric_reporter=MetricsReporter([Channel.STDOUT]),
+        data_provider=data_provider,
+        metrics_reporter=MetricsReporter([Channel.STDOUT]),
     )
 
 
 @hydra.main(config_path=None, config_name="cifar10_tutorial")
-def run(cfg: DictConfig):
+def run(cfg: DictConfig) -> None:
     print(OmegaConf.to_yaml(cfg))
 
     trainer_config = cfg.trainer
