@@ -1,21 +1,22 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
 """In this tutorial, we will train a binary sentiment classifier on LEAF's Sent140 dataset with FLSim.
 
-Before running this file, you need to download the dataset, and partition the data by users. We
-provided the script get_data.sh for such task.
+Before running this file, you need to download the dataset and partition the data by users. We
+provide the script get_data.sh for this purpose.
 
     Typical usage example:
 
     FedAvg
-    python3 sent140_tutorial.py --config-file configs/sent140_config.json
+    python3 sent140_example.py --config-file configs/sent140_config.json
 
     FedBuff + SGDM
-    python3 sent140_tutorial.py --config-file configs/sent140_fedbuff_config.json
+    python3 sent140_example.py --config-file configs/sent140_fedbuff_config.json
 """
 import itertools
 import json
@@ -33,8 +34,8 @@ from flsim.utils.config_utils import maybe_parse_json_config
 from flsim.utils.example_utils import (
     DataProvider,
     FLModel,
-    MetricsReporter,
     LEAFDataLoader,
+    MetricsReporter,
 )
 from hydra.utils import instantiate
 from omegaconf import DictConfig, OmegaConf
@@ -158,7 +159,7 @@ class Sent140Dataset(Dataset):
         return list(itertools.chain.from_iterable(nested_list))
 
 
-def build_data_provider(data_config, drop_last=False):
+def build_data_provider(data_config, drop_last: bool = False):
 
     train_dataset = Sent140Dataset(
         data_root="leaf/data/sent140/data/train/all_data_0_01_keep_1_train_9.json",
@@ -185,9 +186,9 @@ def main_worker(
     trainer_config,
     model_config,
     data_config,
-    use_cuda_if_available=True,
-    distributed_world_size=1,
-):
+    use_cuda_if_available: bool = True,
+    distributed_world_size: int = 1,
+) -> None:
     data_provider, num_letters = build_data_provider(data_config)
 
     model = CharLSTM(
@@ -200,6 +201,7 @@ def main_worker(
     )
     cuda_enabled = torch.cuda.is_available() and use_cuda_if_available
     device = torch.device(f"cuda:{0}" if cuda_enabled else "cpu")
+    # pyre-fixme[6]: Expected `Optional[str]` for 2nd param but got `device`.
     global_model = FLModel(model, device)
     if cuda_enabled:
         global_model.fl_cuda()
@@ -209,19 +211,19 @@ def main_worker(
 
     final_model, eval_score = trainer.train(
         data_provider=data_provider,
-        metric_reporter=metrics_reporter,
-        num_total_users=data_provider.num_users(),
+        metrics_reporter=metrics_reporter,
+        num_total_users=data_provider.num_train_users(),
         distributed_world_size=distributed_world_size,
     )
 
     trainer.test(
-        data_iter=data_provider.test_data(),
-        metric_reporter=MetricsReporter([Channel.STDOUT]),
+        data_provider=data_provider,
+        metrics_reporter=MetricsReporter([Channel.STDOUT]),
     )
 
 
-@hydra.main(config_path=None, config_name="sent140_tutorial")
-def run(cfg: DictConfig):
+@hydra.main(config_path=None, config_name="sent140_config")
+def run(cfg: DictConfig) -> None:
     print(OmegaConf.to_yaml(cfg))
 
     trainer_config = cfg.trainer
