@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -8,6 +9,7 @@ from hydra.core.config_store import ConfigStore
 
 from .base_channel import FLChannelConfig
 from .half_precision_channel import HalfPrecisionChannelConfig
+from .product_quantization_channel import ProductQuantizationChannelConfig
 from .scalar_quantization_channel import ScalarQuantizationChannelConfig
 from .sparse_mask_channel import SparseMaskChannelConfig
 
@@ -34,3 +36,9 @@ ConfigStore.instance().store(
     node=ScalarQuantizationChannelConfig,
     group="channel",
 )
+
+ConfigStore.instance().store(
+    name="base_product_quantization_channel",
+    node=ProductQuantizationChannelConfig,
+    group="channel",
+)
