@@ -1,20 +1,18 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
-from copy import deepcopy
 from typing import Type
 
 import pytest
 from flsim.channels.base_channel import FLChannelConfig, IdentityChannel
-from flsim.channels.communication_stats import (
-    ChannelDirection,
-)
+from flsim.channels.communication_stats import ChannelDirection
 from flsim.channels.message import Message
 from flsim.common.pytest_helper import assertEqual, assertIsInstance
-from flsim.tests import utils
+from flsim.utils import test_utils as utils
 from flsim.utils.fl.common import FLModelParamUtils
 from hydra.utils import instantiate
 
@@ -57,7 +55,7 @@ class TestIdentityChannel:
         # create dummy model
         two_fc = utils.TwoFC()
         base_model = utils.SampleNet(two_fc)
-        download_model = deepcopy(base_model)
+        download_model = FLModelParamUtils.clone(base_model)
 
         # test server -> client, models should be strictly identical
         message = Message(download_model)
@@ -88,7 +86,7 @@ class TestIdentityChannel:
         # create dummy model
         two_fc = utils.TwoFC()
         base_model = utils.SampleNet(two_fc)
-        upload_model = deepcopy(base_model)
+        upload_model = FLModelParamUtils.clone(base_model)
 
         # test client -> server, models should be strictly identical
         message = Message(upload_model)
@@ -117,9 +115,8 @@ class TestIdentityChannel:
 
         # create dummy model
         two_fc = utils.TwoFC()
-        base_model = utils.SampleNet(two_fc)
-        download_model = deepcopy(base_model)
-        upload_model = deepcopy(base_model)
+        download_model = utils.SampleNet(FLModelParamUtils.clone(two_fc))
+        upload_model = utils.SampleNet(FLModelParamUtils.clone(two_fc))
 
         # server -> client
         message = Message(download_model)
