@@ -1,10 +1,10 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
-from copy import deepcopy
 from typing import Any, Optional, Tuple
 
 from flsim.channels.base_channel import IdentityChannel
@@ -21,13 +21,9 @@ from flsim.utils.async_trainer.device_state import (
     TrainingScheduleFactory,
     TrainingState,
 )
-from flsim.utils.async_trainer.training_event_generator import (
-    IEventGenerator,
-)
-from flsim.utils.cuda import (
-    ICudaStateManager,
-    DEFAULT_CUDA_MANAGER,
-)
+from flsim.utils.async_trainer.training_event_generator import IEventGenerator
+from flsim.utils.cuda import DEFAULT_CUDA_MANAGER, ICudaStateManager
+from flsim.utils.fl.common import FLModelParamUtils
 from omegaconf import OmegaConf
 
 
@@ -45,7 +41,7 @@ class AsyncClientFactory:
     ):
         user_info = user_selector.get_random_user()
         training_schedule = TrainingScheduleFactory.create(
-            current_time, event_generator, user_info.user_data.num_examples()
+            current_time, event_generator, user_info.user_data.num_train_examples()
         )
         client = Client(
             **OmegaConf.structured(client_config),
@@ -90,7 +86,7 @@ class AsyncClientDevice(DeviceState):
         self.model_seqnum = model_seqnum
 
     def train_local_model(
-        self, metric_reporter: Any = None
+        self, metrics_reporter: Any = None
     ) -> Tuple[IFLModel, IFLModel, float]:
         r"""
         Performs local training loop
@@ -99,14 +95,14 @@ class AsyncClientDevice(DeviceState):
             self.local_model is not None
         ), "Client has not started training, local_model is None"
         # 1. Save the init model to compute delta
-        before_train_local = deepcopy(self.local_model)
+        before_train_local = FLModelParamUtils.clone(self.local_model)
         # 2. Get ready for training
         self.local_model, optim, optim_scheduler = self.client.prepare_for_training(
             self.local_model
         )
         # 3. Train model on local data
         after_train_local, weight = self.client.train(
-            self.local_model, optim, optim_scheduler, metric_reporter
+            self.local_model, optim, optim_scheduler, metrics_reporter
         )
         # 3. Compute delta
         delta = self.client.compute_delta(
