@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -20,9 +21,8 @@ from flsim.common.timeout_simulator import TimeOutSimulator
 from flsim.data.data_provider import IFLUserData
 from flsim.interfaces.model import IFLModel
 from flsim.privacy.common import PrivacyBudget, PrivacySetting
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
-from flsim.utils.cuda import ICudaStateManager, DEFAULT_CUDA_MANAGER
+from flsim.utils.config_utils import fullclassname, init_self_cfg
+from flsim.utils.cuda import DEFAULT_CUDA_MANAGER, ICudaStateManager
 from opacus import GradSampleModule
 from opacus.accountants import RDPAccountant
 from opacus.optimizers import DPOptimizer
@@ -73,11 +73,11 @@ class DPClient(Client):
         pass
 
     def _get_dataset_stats(self, model: IFLModel):
-        for batch in self.dataset:
+        batch_size = 0
+        for batch in self.dataset.train_data():
             batch_size = model.get_num_examples(batch)
             break
-        # pyre-fixme[61]: `batch_size` may not be initialized here.
-        return batch_size, self.dataset.num_examples()
+        return batch_size, self.dataset.num_train_examples()
 
     @property
     def privacy_budget(self) -> PrivacyBudget:
@@ -98,7 +98,6 @@ class DPClient(Client):
             # pyre-fixme[16]: `DPClient` has no attribute `cfg`.
             if self.cfg.privacy_setting.noise_seed is not None:
                 generator = torch.Generator()
-                # pyre-fixme[16]
                 generator.manual_seed(self.cfg.privacy_setting.noise_seed)
             else:
                 generator = None
@@ -135,7 +134,7 @@ class DPClient(Client):
     ):
         if self.privacy_on and sample_count > optimizer.expected_batch_size:
             raise ValueError(
-                "Batchsize was not properly calculated!"
+                "Batch size was not properly calculated!"
                 " Calculated Epsilons are not Correct"
             )
 
