@@ -1,24 +1,25 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
-"""Contains helper for Python PyTest tests.
+"""Contains helpers for Python PyTest tests.
 
-There are a list of helper utilities missing in PyTest.
+There is a list of helper utilities missing in PyTest.
 We define some of the helpful utilities in this one file
 so that all FL-sim PyTests can reuse them easily.
 """
-from typing import Any, Sized, List
+from typing import Any, List, Sized
 
 
 def assertIsInstance(o: object, t: type) -> None:
     assert isinstance(o, t)
 
 
-def assertAlmostEqual(first, second, places=None, msg=None, delta=None):
-    """Assert that ``first`` and ``second`` is almost equal to each other.
+def assertAlmostEqual(first, second, places=None, msg=None, delta=None) -> None:
+    """Assert that ``first`` and ``second`` are almost equal to each other.
 
     The equality of ``first`` and ``second`` is determined in a similar way to
     the ``assertAlmostEqual`` function of the standard library.
@@ -26,6 +27,7 @@ def assertAlmostEqual(first, second, places=None, msg=None, delta=None):
     if delta is not None and places is not None:
         raise TypeError("specify delta or places - not both")
 
+    # pyre-fixme[35]: Target cannot be annotated.
     msg: str = ""
     diff = abs(second - first)
     if delta is not None:
