@@ -1,4 +1,5 @@
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -12,14 +13,11 @@ from flsim.common.diversity_metrics import (
     DiversityMetricType,
     DiversityStatistics,
 )
-from flsim.common.pytest_helper import (
-    assertEqual,
-    assertTrue,
-)
+from flsim.common.pytest_helper import assertEqual, assertTrue
 
 
 class TestDiversityStatistics:
-    def test_initialize_diversity_statistics(self):
+    def test_initialize_diversity_statistics(self) -> None:
         n_test_cases = 1000
         num_cohorts_per_test = 10
         std_dev = 500.0
@@ -93,7 +91,7 @@ class TestDiversityStatistics:
 
 
 class TestDiversityMetrics:
-    def test_initialize_diversity_metrics(self):
+    def test_initialize_diversity_metrics(self) -> None:
         diversity_metrics = DiversityMetrics(norm_of_sum=1.0, sum_of_norms=2.0)
         assertTrue(isinstance(diversity_metrics, DiversityMetrics))
         assertEqual(
@@ -166,7 +164,7 @@ class TestDiversityMetrics:
                 DiversityMetricType.gradient_diversity
             )
 
-            # Check that metric_value returns the value of the poperty of interest
+            # Check that metric_value returns the value of the property of interest
             diversity_metrics.diversity_metric_type = DiversityMetricType.orthogonality
             assertTrue(
                 math.isclose(
