@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -10,8 +11,7 @@ import abc
 from dataclasses import dataclass
 from typing import List
 
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
+from flsim.utils.config_utils import fullclassname, init_self_cfg
 from flsim.utils.timing.training_duration_distribution import (
     PerExampleGaussianDurationDistributionConfig,
 )
@@ -203,13 +203,13 @@ class GaussianTimeOutSimulator(TimeOutSimulator):
         # update sample second moment
         self._sample_sec_moment_per_user = (
             self._sample_sec_moment_per_user * (self._num_users_tracked - 1)
-            + one_user_training_time ** 2
+            + one_user_training_time**2
         ) / self._num_users_tracked
         # update sample variance, with degree of freedom correction
         if self._num_users_tracked > 1:
             self._sample_var_per_user = (
                 self._num_users_tracked / (self._num_users_tracked - 1)
-            ) * (self._sample_sec_moment_per_user - self._sample_mean_per_user ** 2)
+            ) * (self._sample_sec_moment_per_user - self._sample_mean_per_user**2)
 
     def user_timeout(self, training_time: float) -> bool:
         """
