@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -20,9 +21,7 @@ from torch.utils.tensorboard import SummaryWriter
 
 
 class FLMetricsReporter(IFLMetricsReporter, abc.ABC):
-    """
-    This is a MetricsReporter with Tensorboard support.
-    """
+    """MetricsReporter with Tensorboard support."""
 
     def __init__(self, channels: List[Channel], log_dir: Optional[str] = None):
         self.channels = channels
@@ -36,7 +35,7 @@ class FLMetricsReporter(IFLMetricsReporter, abc.ABC):
         self.predictions_list = []
         self.targets_list = []
         self.model_inputs_list = []
-        self.latest_scores: Dict[str, Any] = {}
+        self.latest_scores = {}
         self.best_eval_metrics = None
 
     def set_summary_writer(self, log_dir: Optional[str]):
@@ -100,6 +99,8 @@ class FLMetricsReporter(IFLMetricsReporter, abc.ABC):
                     timeline.global_round_num(),
                 )
 
+            # Score is usually a more interpretable metric than loss and higher is better
+            # For classification tasks, accuracy is a typical score
             scores = self.compute_scores()
             self.latest_scores = scores
 
@@ -114,10 +115,13 @@ class FLMetricsReporter(IFLMetricsReporter, abc.ABC):
                         score,
                         timeline.global_round_num(),
                     )
+
+            # Construct evaluation metric object
             eval_metrics = self.create_eval_metrics(
                 scores, mean_loss, timeline=timeline, stage=stage
             )
-        # handle misc reporting values
+
+        # Miscellaneous metrics beyond loss and score
         metrics = extra_metrics or []
         for metric in metrics:
             value = Metric.to_dict(metric.value) if metric.is_compund else metric.value
@@ -172,12 +176,13 @@ class FLMetricsReporter(IFLMetricsReporter, abc.ABC):
     def create_eval_metrics(
         self, scores: Dict[str, Any], total_loss: float, **kwargs
     ) -> Any:
-        """One should provide concrete implementation of how to construct
-        object that represents evaluation metrics based on scores and total
-        loss. Most of the case, one would just pick one of the scores or
-        total loss as the evaluation metrics to pick the better model, but
-        this interface also allows s/he to make evaluation metrics more
-        complex and use it in conjunction with compare_metrics() function
-        to determine which metrics is the better one.
+        """One should provide a concrete implementation of how to construct
+        an object that represents evaluation metrics based on scores and
+        total loss. Usually, one would just pick one of the scores or
+        total loss as the evaluation metric to pick the better model, but
+        this interface also allows one to make evaluation metrics more
+        complex and use them in conjunction with the compare_metrics()
+        function to determine which metrics and corresponding model are
+        better.
         """
         pass
