@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -8,16 +9,20 @@ from hydra.core.config_store import ConfigStore  #  @manual
 
 from .async_aggregators import (
     FedAdamAsyncAggregatorConfig,
+    FedAdamFedBuffAggregatorConfig,
     FedAvgWithLRAsyncAggregatorConfig,
+    FedAvgWithLRFedBuffAggregatorConfig,
     FedAvgWithLRWithMomentumAsyncAggregatorConfig,
-    FedAdamHybridAggregatorConfig,
-    FedAvgWithLRHybridAggregatorConfig,
 )
-from .local_optimizers import LocalOptimizerSGDConfig, LocalOptimizerFedProxConfig
+from .local_optimizers import (
+    LocalOptimizerAdamConfig,
+    LocalOptimizerFedProxConfig,
+    LocalOptimizerSGDConfig,
+)
 from .optimizer_scheduler import (
+    ArmijoLineSearchSchedulerConfig,
     ConstantLRSchedulerConfig,
     LRBatchSizeNormalizerSchedulerConfig,
-    ArmijoLineSearchSchedulerConfig,
 )
 from .server_optimizers import (
     FedAdamOptimizerConfig,
@@ -25,13 +30,15 @@ from .server_optimizers import (
     FedAvgWithLROptimizerConfig,
     FedLAMBOptimizerConfig,
     FedLARSOptimizerConfig,
+    ServerFTRLOptimizerConfig,
 )
+
 from .sync_aggregators import (
-    FedAvgSyncAggregatorConfig,
     FedAdamSyncAggregatorConfig,
+    FedAvgSyncAggregatorConfig,
     FedAvgWithLRSyncAggregatorConfig,
-    FedLARSSyncAggregatorConfig,
     FedLAMBSyncAggregatorConfig,
+    FedLARSSyncAggregatorConfig,
 )
 
 ConfigStore.instance().store(
@@ -47,6 +54,12 @@ ConfigStore.instance().store(
     group="optimizer",
 )
 
+ConfigStore.instance().store(
+    name="base_optimizer_adam",
+    node=LocalOptimizerAdamConfig,
+    group="optimizer",
+)
+
 ConfigStore.instance().store(
     name="base_constant_lr_scheduler",
     node=ConstantLRSchedulerConfig,
@@ -125,13 +138,13 @@ ConfigStore.instance().store(
 
 ConfigStore.instance().store(
     name="base_fed_avg_with_lr_hybrid_aggregator",
-    node=FedAvgWithLRHybridAggregatorConfig,
+    node=FedAvgWithLRFedBuffAggregatorConfig,
     group="aggregator",
 )
 
 ConfigStore.instance().store(
     name="base_fed_adam_hybrid_aggregator",
-    node=FedAdamHybridAggregatorConfig,
+    node=FedAdamFedBuffAggregatorConfig,
     group="aggregator",
 )
 
@@ -167,3 +180,9 @@ ConfigStore.instance().store(
     node=FedLAMBOptimizerConfig,
     group="server_optimizer",
 )
+
+ConfigStore.instance().store(
+    name="base_ftrl_optimizer",
+    node=ServerFTRLOptimizerConfig,
+    group="server_optimizer",
+)
