@@ -1,12 +1,12 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
 from __future__ import annotations
 
-import copy
 import logging
 from dataclasses import dataclass
 from enum import Enum
@@ -17,14 +17,10 @@ from flsim.channels.base_channel import IdentityChannel
 from flsim.common.logger import Logger
 from flsim.interfaces.model import IFLModel
 from flsim.privacy.common import PrivacyBudget
-from flsim.reducers.base_round_reducer import (
-    ReductionType,
-    RoundReducerConfig,
-)
+from flsim.reducers.base_round_reducer import ReductionType, RoundReducerConfig
 from flsim.reducers.dp_round_reducer import DPRoundReducerConfig
 from flsim.reducers.weighted_dp_round_reducer import WeightedDPRoundReducerConfig
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
+from flsim.utils.config_utils import fullclassname, init_self_cfg
 from flsim.utils.fl.common import FLModelParamUtils
 from flsim.utils.fl.stats import ModelSequenceNumberTracker
 from hydra.utils import instantiate
@@ -112,7 +108,7 @@ class AsyncAggregator:
 
         self.orig_lr = self.optimizer.param_groups[0]["lr"]
         self._global_model: IFLModel = global_model
-        self._reconstructed_grad: IFLModel = copy.deepcopy(self._global_model)
+        self._reconstructed_grad: IFLModel = FLModelParamUtils.clone(self._global_model)
         # there is no concept of a round in async, hence round reducer is not tied to a round
         self.reducer = instantiate(
             self.cfg.reducer,
@@ -335,14 +331,14 @@ class FedAvgWithLRWithMomentumAsyncAggregator(FedAvgWithLRAsyncAggregator):
         return True
 
 
-class HybridAggregator(AsyncAggregator):
+class FedBuffAggregator(AsyncAggregator):
     r"""
-    Aggregator for Hybrid-FL where client update and global update are decoupled
+    Aggregator for FedBuff (Buffered Asynchronous Aggregation)
 
     Keeps track of number clients reported and take a global step after reaching the
     threshold set by the config
     """
-    logger: logging.Logger = Logger.get_logger("HybridAggregator")
+    logger: logging.Logger = Logger.get_logger("FedBuffAggregator")
 
     def __init__(
         self,
@@ -354,14 +350,14 @@ class HybridAggregator(AsyncAggregator):
         init_self_cfg(
             self,
             component_class=__class__,
-            config_class=HybridAggregatorConfig,
+            config_class=FedBuffAggregatorConfig,
             **kwargs,
         )
 
         super().__init__(global_model=global_model, channel=channel, **kwargs)
-        # pyre-fixme[16]: `HybridAggregator` has no attribute `cfg`.
+        # pyre-fixme[16]: `FedBuffAggregator` has no attribute `cfg`.
         if self.cfg.aggregation_type != AsyncAggregationType.fed_buff_aggregation:
-            raise ValueError("Hybrid Aggregator only supports delta direction")
+            raise ValueError("Buffered Aggregator only supports delta direction")
         self.num_clients_reported = 0
 
     @classmethod
@@ -370,7 +366,7 @@ class HybridAggregator(AsyncAggregator):
 
     def zero_grad(self):
         r"""
-        Zero'd out the grads if it's the first update or if
+        Zeroes out the grads if it's the first update or if
         reaching buffer_size otherwise no-op
         """
         if self.num_clients_reported == 0 or self.should_update_global_model():
@@ -401,7 +397,7 @@ class HybridAggregator(AsyncAggregator):
         return False
 
     def should_update_global_model(self) -> bool:
-        # pyre-fixme[16]: `HybridAggregator` has no attribute `cfg`.
+        # pyre-fixme[16]: `FedBuffAggregator` has no attribute `cfg`.
         return self.num_clients_reported >= self.cfg.buffer_size
 
     def on_training_epoch_end(self) -> bool:
@@ -429,14 +425,14 @@ class HybridAggregator(AsyncAggregator):
 
     def _collect_client_update(self, update: IFLModel, weight: float) -> None:
         """
-        Collects update from one client and aggregtes it internally.
+        Collects update from one client and aggregates it internally.
         reduced model = reduced model + update * weight
         """
         self.reducer.collect_update(delta=update, weight=weight)
         self.num_clients_reported += 1
 
 
-class FedAvgWithLRHybridAggregator(HybridAggregator):
+class FedAvgWithLRFedBuffAggregator(FedBuffAggregator):
     def __init__(
         self,
         *,
@@ -447,7 +443,7 @@ class FedAvgWithLRHybridAggregator(HybridAggregator):
         init_self_cfg(
             self,
             component_class=__class__,
-            config_class=FedAvgWithLRHybridAggregatorConfig,
+            config_class=FedAvgWithLRFedBuffAggregatorConfig,
             **kwargs,
         )
 
@@ -458,7 +454,7 @@ class FedAvgWithLRHybridAggregator(HybridAggregator):
         pass
 
 
-class FedAdamHybridAggregator(HybridAggregator):
+class FedAdamFedBuffAggregator(FedBuffAggregator):
     def __init__(
         self,
         *,
@@ -469,7 +465,7 @@ class FedAdamHybridAggregator(HybridAggregator):
         init_self_cfg(
             self,
             component_class=__class__,
-            config_class=FedAdamHybridAggregatorConfig,
+            config_class=FedAdamFedBuffAggregatorConfig,
             **kwargs,
         )
 
@@ -483,7 +479,7 @@ class FedAdamHybridAggregator(HybridAggregator):
 def create_optimizer_for_async_aggregator(config: AsyncAggregatorConfig, model: Model):
     if config._target_ in {
         FedAvgWithLRAsyncAggregatorConfig._target_,
-        FedAvgWithLRHybridAggregatorConfig._target_,
+        FedAvgWithLRFedBuffAggregatorConfig._target_,
         FedAvgWithLRWithMomentumAsyncAggregatorConfig._target_,
     }:
         return torch.optim.SGD(
@@ -495,7 +491,7 @@ def create_optimizer_for_async_aggregator(config: AsyncAggregatorConfig, model:
         )
     elif config._target_ in {
         FedAdamAsyncAggregatorConfig._target_,
-        FedAdamHybridAggregatorConfig._target_,
+        FedAdamFedBuffAggregatorConfig._target_,
     }:
         return torch.optim.Adam(
             model.parameters(),
@@ -547,21 +543,21 @@ class FedAdamAsyncAggregatorConfig(AsyncAggregatorConfig):
 
 
 @dataclass
-class HybridAggregatorConfig(AsyncAggregatorConfig):
+class FedBuffAggregatorConfig(AsyncAggregatorConfig):
     # number of clients to collect before taking a global step
     buffer_size: int = 1
 
 
 @dataclass
-class FedAvgWithLRHybridAggregatorConfig(HybridAggregatorConfig):
-    _target_: str = fullclassname(FedAvgWithLRHybridAggregator)
+class FedAvgWithLRFedBuffAggregatorConfig(FedBuffAggregatorConfig):
+    _target_: str = fullclassname(FedAvgWithLRFedBuffAggregator)
     lr: float = 0.001
     momentum: float = 0.0
 
 
 @dataclass
-class FedAdamHybridAggregatorConfig(HybridAggregatorConfig):
-    _target_: str = fullclassname(FedAdamHybridAggregator)
+class FedAdamFedBuffAggregatorConfig(FedBuffAggregatorConfig):
+    _target_: str = fullclassname(FedAdamFedBuffAggregator)
     lr: float = 0.001
     weight_decay: float = 0.00001
     beta1: float = 0.9
