@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -10,8 +11,7 @@ from dataclasses import dataclass
 from typing import Any, Dict
 
 import torch
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
+from flsim.utils.config_utils import fullclassname, init_self_cfg
 from omegaconf import MISSING
 from torch.nn import Module as Model  # @manual
 
@@ -31,7 +31,6 @@ class LocalOptimizer:
         pass
 
 
-# pyre-ignore[11] Annotation `torch.optim.SGD` is not defined as a type
 class LocalOptimizerSGD(LocalOptimizer, torch.optim.SGD):
     def __init__(self, *, model: Model, **kwargs) -> None:
         init_self_cfg(
@@ -72,6 +71,28 @@ class LocalOptimizerSGD(LocalOptimizer, torch.optim.SGD):
         }
 
 
+class LocalOptimizerAdam(LocalOptimizer, torch.optim.Adam):
+    def __init__(self, *, model: Model, **kwargs) -> None:
+        init_self_cfg(
+            self,
+            component_class=__class__,
+            config_class=LocalOptimizerAdamConfig,
+            **kwargs,
+        )
+
+        super().__init__(model=model, **kwargs)
+
+        torch.optim.Adam.__init__(
+            self,
+            params=self.model.parameters(),
+            # pyre-ignore[16] Undefined attribute
+            lr=self.cfg.lr,
+            weight_decay=self.cfg.weight_decay,
+            betas=(self.cfg.beta1, self.cfg.beta2),
+            eps=self.cfg.eps,
+        )
+
+
 class LocalOptimizerFedProx(LocalOptimizer, torch.optim.SGD):
     def __init__(
         self,
@@ -162,6 +183,16 @@ class LocalOptimizerSGDConfig(LocalOptimizerConfig):
     _target_: str = fullclassname(LocalOptimizerSGD)
 
 
+@dataclass
+class LocalOptimizerAdamConfig(LocalOptimizerConfig):
+    _target_: str = fullclassname(LocalOptimizerAdam)
+    lr: float = 0.001
+    weight_decay: float = 0.00001
+    beta1: float = 0.9
+    beta2: float = 0.999
+    eps: float = 1e-8
+
+
 @dataclass
 class LocalOptimizerFedProxConfig(LocalOptimizerConfig):
     _target_: str = fullclassname(LocalOptimizerFedProx)
