@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -13,8 +14,7 @@ from typing import Any, Optional
 
 from flsim.interfaces.batch_metrics import IFLBatchMetrics
 from flsim.interfaces.model import IFLModel
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
+from flsim.utils.config_utils import fullclassname, init_self_cfg
 from flsim.utils.fl.common import FLModelParamUtils
 from omegaconf import MISSING
 from torch.optim.optimizer import Optimizer
@@ -22,7 +22,7 @@ from torch.optim.optimizer import Optimizer
 
 class OptimizerScheduler(abc.ABC):
     """
-    base class for local LR scheduler, enable the laerning rate
+    Base class for local LR scheduler, enable the learning rate
     of local optimizers for individual users during local training
     """
 
@@ -53,8 +53,8 @@ class OptimizerScheduler(abc.ABC):
         epoch: Optional[int] = None,
     ):
         """
-        interface for updating learning rate. Some learning rate scheduling methods
-        rely produces multiple trial forward passes internally, e.g, line search
+        Interface for updating learning rate. Some learning rate scheduling methods
+        rely on producing multiple trial forward passes internally, e.g, line search
         methods, hence model is required in the interface.
         """
         pass
@@ -100,7 +100,7 @@ class ConstantLRScheduler(OptimizerScheduler):
 
 class LRBatchSizeNormalizer(OptimizerScheduler):
     """
-    normalized the LR by number of examples in the batch
+    Normalize the LR by number of examples in the batch
     """
 
     def __init__(
@@ -145,7 +145,7 @@ class LRBatchSizeNormalizer(OptimizerScheduler):
 class ArmijoLineSearch(OptimizerScheduler):
     """
     Classical Armijo line-search for step-size selection in optimization.
-    Recent work suggests that it might also be used in stochastic over-parametrized
+    Recent work suggests that it might also be used in stochastic over-parameterized
     setting, cf.
     "Painless Stochastic Gradient: Interpolation, Line-Search, and Convergence Rates"
     """
@@ -212,7 +212,6 @@ class ArmijoLineSearch(OptimizerScheduler):
             assert (
                 len(proposed_lr) == 1
             ), "Armijo line-search only works with single param_group"
-            # pyre-ignore[20]
             self.optimizer.step()
             # DO NOT compute backprop after forward here, only the forward is
             # required for step-size selection, use existent gradient direction
@@ -222,7 +221,7 @@ class ArmijoLineSearch(OptimizerScheduler):
             if (
                 float(new_loss)
                 <= loss_before_update
-                - self.cfg.c * proposed_lr[0] * grad_norm_before_update ** 2
+                - self.cfg.c * proposed_lr[0] * grad_norm_before_update**2
             ):
                 # satisfy sufficient descent, accept proposed_lr
                 # and terminate line search
@@ -230,7 +229,7 @@ class ArmijoLineSearch(OptimizerScheduler):
             # reduce lr
             self._shrink_lr()
 
-        # recover model state before the line search scatching, do the actual
+        # recover model state before the line search started, do the actual
         # optimizer.step() outside of the scheduler
         FLModelParamUtils.load_state_dict(
             model.fl_get_module(), state_dict, only_federated_params=False
