@@ -1,10 +1,10 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
-import copy
 import random
 from dataclasses import dataclass
 from typing import List
@@ -12,20 +12,23 @@ from typing import List
 import numpy as np
 import pytest
 import torch
-from flsim.common.pytest_helper import assertTrue, assertEqual, assertEmpty
+from flsim.common.pytest_helper import assertEmpty, assertEqual, assertTrue
 from flsim.interfaces.model import IFLModel
 from flsim.optimizers.async_aggregators import (
     AsyncAggregationType,
     AsyncAggregatorConfig,
     FedAvgWithLRAsyncAggregatorConfig,
+    FedAvgWithLRFedBuffAggregatorConfig,
     FedAvgWithLRWithMomentumAsyncAggregatorConfig,
-    FedAvgWithLRHybridAggregatorConfig,
-)
-from flsim.tests.utils import MockQuadratic1DFL, Quadratic1D, SampleNet, TwoFC
-from flsim.tests.utils import (
-    verify_models_equivalent_after_training,
 )
 from flsim.utils.fl.common import FLModelParamUtils
+from flsim.utils.test_utils import (
+    MockQuadratic1DFL,
+    Quadratic1D,
+    SampleNet,
+    TwoFC,
+    verify_models_equivalent_after_training,
+)
 from hydra.utils import instantiate
 
 
@@ -43,7 +46,7 @@ class TestAsyncAggregator:
         param_after_global_training: float,
         weight: float,
         config: AsyncAggregatorConfig,
-    ):
+    ) -> None:
         """
         Test async aggregator by:
         1. Create AsyncAggregator of given type (fed_buff_aggregation or fed_async_aggregation)
@@ -56,8 +59,8 @@ class TestAsyncAggregator:
         init_val = 1
         global_model = MockQuadratic1DFL(Quadratic1D())
         async_aggregator = instantiate(config, global_model=global_model)
-        local_model = copy.deepcopy(global_model)
-        delta = copy.deepcopy(global_model)
+        local_model = FLModelParamUtils.clone(global_model)
+        delta = FLModelParamUtils.clone(global_model)
         # local training, x becomes param_after_local_training
         local_model.fl_get_module().x.data = torch.Tensor([param_after_local_training])
         # global update from another user, x becomes param_after_global_training
@@ -94,13 +97,13 @@ class TestAsyncAggregator:
             )
         )
 
-    def test_fed_buff_aggregation(self):
+    def test_fed_buff_aggregation(self) -> None:
         """
         Test that AsyncAggregator with 'fed_buff_aggregation' works as expected
         1. Create global_model, some_param=1
         2. Copy global_model into local_model
         3. Simulate local training by local_model changing some_param to a (some_param=a, delta=a-1)
-        4. Simulate global model updata by other user, some_param being set to b (some_param=b)
+        4. Simulate global model update by another user, some_param being set to b (some_param=b)
         5. Call AsyncOptimizer.step(). global_model.some_param should be b + (a-1) (some_param == b + a -1)
         """
 
@@ -110,7 +113,7 @@ class TestAsyncAggregator:
             FedAvgWithLRAsyncAggregatorConfig(
                 aggregation_type=AsyncAggregationType.fed_buff_aggregation, lr=1.0
             ),
-            FedAvgWithLRHybridAggregatorConfig(
+            FedAvgWithLRFedBuffAggregatorConfig(
                 aggregation_type=AsyncAggregationType.fed_buff_aggregation,
                 lr=1.0,
                 buffer_size=1,
@@ -125,13 +128,13 @@ class TestAsyncAggregator:
                         config=config,
                     )
 
-    def test_fed_async_aggregation(self):
+    def test_fed_async_aggregation(self) -> None:
         """
         Test that AsyncAggregator with 'fed_async_aggregation' works as expected
         1. Create global_model, some_param=1
         2. Copy global_model into local_model
         3. Simulate local training by local_model changing some_param to a (some_param=a)
-        4. Simulate global model updata by other user, some_param being set to b (some_param=b)
+        4. Simulate global model update by another user, some_param being set to b (some_param=b)
         5. Call AsyncOptimizer.step(weight=1). global_model.some_param should be
             w*a + (1-w)*b (some_param == w*a + (1-w)*b)
         """
@@ -159,24 +162,24 @@ class TestAsyncAggregator:
             for _ in range(num_clients)
         ]
 
-    def _symmetry_test(self, num_users, hybrid_config):
+    def _symmetry_test(self, num_users, fedbuff_config) -> str:
 
-        hybrid_global_model_1 = SampleNet(TwoFC())
-        hybrid_global_model_2 = copy.deepcopy(hybrid_global_model_1)
+        fedbuff_global_model_1 = SampleNet(TwoFC())
+        fedbuff_global_model_2 = FLModelParamUtils.clone(fedbuff_global_model_1)
 
-        hybrid_aggregator_1 = instantiate(
-            hybrid_config, global_model=hybrid_global_model_1
+        fedbuff_aggregator_1 = instantiate(
+            fedbuff_config, global_model=fedbuff_global_model_1
         )
 
-        hybrid_aggregator_2 = instantiate(
-            hybrid_config, global_model=hybrid_global_model_2
+        fedbuff_aggregator_2 = instantiate(
+            fedbuff_config, global_model=fedbuff_global_model_2
         )
 
         client_models = self._create_n_clients(num_users)
 
         for client_model in client_models:
-            hybrid_aggregator_1.zero_grad()
-            hybrid_aggregator_1.on_client_training_end(
+            fedbuff_aggregator_1.zero_grad()
+            fedbuff_aggregator_1.on_client_training_end(
                 client_model.delta,
                 client_model.after_train,
                 weight=client_model.weight,
@@ -184,8 +187,8 @@ class TestAsyncAggregator:
 
         random.shuffle(client_models)
         for client_model in client_models:
-            hybrid_aggregator_2.zero_grad()
-            hybrid_aggregator_2.on_client_training_end(
+            fedbuff_aggregator_2.zero_grad()
+            fedbuff_aggregator_2.on_client_training_end(
                 client_model.delta,
                 client_model.after_train,
                 weight=client_model.weight,
@@ -193,20 +196,22 @@ class TestAsyncAggregator:
 
         return FLModelParamUtils.get_mismatched_param(
             models=[
-                hybrid_global_model_1.fl_get_module(),
-                hybrid_global_model_2.fl_get_module(),
+                fedbuff_global_model_1.fl_get_module(),
+                fedbuff_global_model_2.fl_get_module(),
             ],
             rel_epsilon=1e-6,
             abs_epsilon=1e-6,
         )
 
-    def _equivalence_test(self, num_users, hybrid_config, async_config):
+    def _equivalence_test(self, num_users, fedbuff_config, async_config) -> str:
         async_global_model = SampleNet(TwoFC())
-        hybrid_global_model = copy.deepcopy(async_global_model)
+        fedbuff_global_model = FLModelParamUtils.clone(async_global_model)
 
         async_aggregator = instantiate(async_config, global_model=async_global_model)
 
-        hybrid_aggregator = instantiate(hybrid_config, global_model=hybrid_global_model)
+        fedbuff_aggregator = instantiate(
+            fedbuff_config, global_model=fedbuff_global_model
+        )
 
         client_models = self._create_n_clients(num_users)
 
@@ -219,8 +224,8 @@ class TestAsyncAggregator:
             )
 
         for client_model in client_models:
-            hybrid_aggregator.zero_grad()
-            hybrid_aggregator.on_client_training_end(
+            fedbuff_aggregator.zero_grad()
+            fedbuff_aggregator.on_client_training_end(
                 client_model.delta,
                 client_model.after_train,
                 weight=client_model.weight,
@@ -229,41 +234,43 @@ class TestAsyncAggregator:
         return FLModelParamUtils.get_mismatched_param(
             models=[
                 async_global_model.fl_get_module(),
-                hybrid_global_model.fl_get_module(),
+                fedbuff_global_model.fl_get_module(),
             ],
             rel_epsilon=1e-6,
             abs_epsilon=1e-6,
         )
 
-    def test_hybrid_async_symmetry(self):
+    def test_fedbuff_async_symmetry(self) -> None:
         """
         Test for symmetry:
-        To satisfy symmetry, a hybrid async aggregation algorithm should be invariant to the order of user updates
+        To satisfy symmetry, a buffered async aggregation algorithm should be invariant to the order of user updates
         f(userA, userB) = f(userB, userA) where f() is aggregation mechanism
 
-        1. Create async and hybrid aggregators with same global model
+        1. Create async and fedbuff aggregators with same global model
         2. Create a list of N clients
-        3. Run hybrid_aggregator
+        3. Run fedbuff_aggregator
         4. Shuffle client list
         5. Run async_aggregator
         6. Both should reach the same final global model
         """
         num_users = 10
         global_lr = 1.0
-        hybrid_config = FedAvgWithLRHybridAggregatorConfig(lr=global_lr, buffer_size=1)
+        fedbuff_config = FedAvgWithLRFedBuffAggregatorConfig(
+            lr=global_lr, buffer_size=1
+        )
 
         error_msg = self._symmetry_test(
-            num_users=num_users, hybrid_config=hybrid_config
+            num_users=num_users, fedbuff_config=fedbuff_config
         )
         assertEmpty(error_msg, msg=error_msg)
 
-    def test_hybrid_async_equivalence(self):
+    def test_fedbuff_async_equivalence(self) -> None:
         """
         To satisfy equivalence,
         1. Assume both mechanisms have the same starting point
         2. Denote N = number of users
         3. Assume buffer_size is a factor of N
-        4. Pure async and hybrid-async would reach the same final global model
+        4. Pure async and fedbuff-async would reach the same final global model
 
         For simplicity, we assume buffer_size = N
         """
@@ -271,14 +278,18 @@ class TestAsyncAggregator:
         global_lr = 1.00
 
         async_config = FedAvgWithLRAsyncAggregatorConfig(lr=global_lr)
-        hybrid_config = FedAvgWithLRHybridAggregatorConfig(lr=global_lr, buffer_size=10)
+        fedbuff_config = FedAvgWithLRFedBuffAggregatorConfig(
+            lr=global_lr, buffer_size=10
+        )
 
         error_msg = self._equivalence_test(
-            num_users=num_users, hybrid_config=hybrid_config, async_config=async_config
+            num_users=num_users,
+            fedbuff_config=fedbuff_config,
+            async_config=async_config,
         )
         assertEmpty(error_msg, msg=error_msg)
 
-    def test_global_update(self):
+    def test_global_update(self) -> None:
         """
         Test the aggregator only updates global model if
         threshold is reached
@@ -288,14 +299,14 @@ class TestAsyncAggregator:
             num_total_users = np.random.randint(1, 20)
             buffer_size = np.random.randint(1, num_total_users + 1)
 
-            hybrid_config = FedAvgWithLRHybridAggregatorConfig(
+            fedbuff_config = FedAvgWithLRFedBuffAggregatorConfig(
                 lr=1.0, buffer_size=buffer_size
             )
             global_model = SampleNet(TwoFC())
-            hybrid_aggregator = instantiate(hybrid_config, global_model=global_model)
+            fedbuff_aggregator = instantiate(fedbuff_config, global_model=global_model)
             client_models = self._create_n_clients(num_total_users)
             for client_num, client in enumerate(client_models):
-                is_global_model_updated = hybrid_aggregator.on_client_training_end(
+                is_global_model_updated = fedbuff_aggregator.on_client_training_end(
                     client.delta, client.after_train, weight=1
                 )
                 # client_num is 0th index hence we need the + 1
@@ -360,8 +371,8 @@ class TestAsyncAggregator:
         [(1, 2, 0.5), (10, 10, 0.5), (10, 10, 0)],
     )
     def test_momentum_implementation_zero_weight(
-        self, num_total_users, num_epochs, momentum
-    ):
+        self, num_total_users: int, num_epochs: int, momentum: float
+    ) -> None:
         """In FedAsyncAggregatorWithMomentum.on_client_training_end, when weight=0,
         neither velocity nor model should be updated
         We test this by comparing two training runs:
@@ -380,7 +391,7 @@ class TestAsyncAggregator:
         torch.manual_seed(1)
         np.random.seed(1)
         global_model_trained1 = self.train_async_with_zero_weight(
-            initial_model=copy.deepcopy(initial_model),
+            initial_model=FLModelParamUtils.clone(initial_model),
             client_models=client_models,
             num_epochs=num_epochs,
             num_total_users=num_total_users,
@@ -390,7 +401,7 @@ class TestAsyncAggregator:
         torch.manual_seed(1)
         np.random.seed(1)
         global_model_trained2 = self.train_async_with_zero_weight(
-            initial_model=copy.deepcopy(initial_model),
+            initial_model=FLModelParamUtils.clone(initial_model),
             client_models=client_models,
             num_epochs=num_epochs,
             num_total_users=num_total_users,
@@ -412,8 +423,8 @@ class TestAsyncAggregator:
         [(1, 2, 0.5, 10), (10, 10, 0.5, 10), (10, 10, 0, 10)],
     )
     def test_momentum_implementation_one_weight(
-        self, num_total_users, num_epochs, momentum, lr
-    ):
+        self, num_total_users: int, num_epochs: int, momentum: float, lr: float
+    ) -> None:
         """FedAsyncAggregatorWithMomentum.on_client_training_end should behave
         exactly like SGD with momentum when weight = 1
         We test this by
@@ -433,7 +444,9 @@ class TestAsyncAggregator:
         torch.manual_seed(1)
         np.random.seed(1)
         config = FedAvgWithLRWithMomentumAsyncAggregatorConfig(lr=lr, momentum=momentum)
-        aggregator = instantiate(config, global_model=copy.deepcopy(initial_model))
+        aggregator = instantiate(
+            config, global_model=FLModelParamUtils.clone(initial_model)
+        )
         for _ in range(num_epochs):
             for client in client_models:
                 aggregator.on_client_training_end(
@@ -443,7 +456,7 @@ class TestAsyncAggregator:
         # run SGD training
         torch.manual_seed(1)
         np.random.seed(1)
-        sgd_model = copy.deepcopy(initial_model)
+        sgd_model = FLModelParamUtils.clone(initial_model)
         sgd_optimizer = torch.optim.SGD(
             sgd_model.fl_get_module().parameters(), lr=lr, momentum=momentum
         )
