@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -10,12 +11,12 @@ from flsim.optimizers.optimizer_scheduler import (
     ArmijoLineSearch,
     ArmijoLineSearchSchedulerConfig,
 )
-from flsim.tests.utils import MockQuadratic1DFL, Quadratic1D
+from flsim.utils.test_utils import MockQuadratic1DFL, Quadratic1D
 from omegaconf import OmegaConf
 
 
 class TestOptimizerScheduler:
-    def test_armijo_line_search_on_parabola(self):
+    def test_armijo_line_search_on_parabola(self) -> None:
         """
             a toy optimization example:
                 min f(x) = 100 x^2 - 1
