@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -7,16 +8,14 @@
 from enum import Enum
 
 from flsim.privacy.privacy_engine import (
-    TreePrivacyEngine,
     GaussianPrivacyEngine,
-    PrivacySetting,
     IPrivacyEngine,
+    PrivacySetting,
 )
 
 
 class NoiseType(Enum):
-    TREE_NOISE = "tree"
-    GAUSSIAN = "guassian"
+    GAUSSIAN = "gaussian"
 
 
 class PrivacyEngineFactory:
@@ -28,15 +27,9 @@ class PrivacyEngineFactory:
         num_total_users: int,
         noise_type: NoiseType,
     ) -> IPrivacyEngine:
-        if noise_type == NoiseType.TREE_NOISE:
-            return TreePrivacyEngine(
-                privacy_setting=privacy_setting,
-                users_per_round=users_per_round,
-                num_total_users=num_total_users,
-            )
-        else:
-            return GaussianPrivacyEngine(
-                privacy_setting=privacy_setting,
-                users_per_round=users_per_round,
-                num_total_users=num_total_users,
-            )
+        assert noise_type == NoiseType.GAUSSIAN
+        return GaussianPrivacyEngine(
+            privacy_setting=privacy_setting,
+            users_per_round=users_per_round,
+            num_total_users=num_total_users,
+        )
