@@ -1,10 +1,10 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
-import copy
 from typing import Any, Dict, Optional
 
 import numpy as np
@@ -15,8 +15,8 @@ from flsim.active_user_selectors.simple_user_selector import (
 )
 from flsim.clients.base_client import ClientConfig
 from flsim.clients.dp_client import DPClientConfig
-from flsim.common.pytest_helper import assertEqual, assertEmpty, assertNotEmpty
-from flsim.optimizers.async_aggregators import FedAvgWithLRHybridAggregatorConfig
+from flsim.common.pytest_helper import assertEmpty, assertEqual, assertNotEmpty
+from flsim.optimizers.async_aggregators import FedAvgWithLRFedBuffAggregatorConfig
 from flsim.optimizers.local_optimizers import LocalOptimizerSGDConfig
 from flsim.privacy.common import PrivacySetting
 from flsim.reducers.base_round_reducer import ReductionType
@@ -24,18 +24,18 @@ from flsim.reducers.weighted_dp_round_reducer import WeightedDPRoundReducerConfi
 from flsim.servers.aggregator import AggregationType
 from flsim.servers.sync_dp_servers import SyncDPSGDServerConfig
 from flsim.servers.sync_servers import SyncServerConfig
-from flsim.tests.utils import (
-    MetricsReporterWithMockedChannels,
-    FakeMetricReporter,
-    verify_models_equivalent_after_training,
-)
 from flsim.trainers.sync_trainer import SyncTrainer, SyncTrainerConfig
 from flsim.utils.async_trainer.async_staleness_weights import (
     PolynomialStalenessWeightConfig,
 )
 from flsim.utils.fl.common import FLModelParamUtils
 from flsim.utils.sample_model import DummyAlphabetFLModel
-from flsim.utils.tests.helpers.async_trainer_test_utils import (
+from flsim.utils.test_utils import (
+    FakeMetricReporter,
+    MetricsReporterWithMockedChannels,
+    verify_models_equivalent_after_training,
+)
+from flsim.utils.tests.helpers.test_async_trainer_utils import (
     get_fl_data_provider,
     run_fl_training,
 )
@@ -51,8 +51,8 @@ class TestDifferentialPrivacyIntegration:
         noise,
         clipping,
         reduction_type=ReductionType.WEIGHTED_SUM,
-        min_weight=1e-6,
-        max_weight=1.0,
+        min_weight: float = 1e-6,
+        max_weight: float = 1.0,
     ):
         return WeightedDPRoundReducerConfig(
             reduction_type=reduction_type,
@@ -106,7 +106,7 @@ class TestDifferentialPrivacyIntegration:
             dummy_dataset, shard_size, local_batch_size, DummyAlphabetFLModel()
         )
         assertEqual(data_loader.num_total_users, data_size / shard_size)
-        assertEqual(data_loader.num_total_users, data_provider.num_users())
+        assertEqual(data_loader.num_total_users, data_provider.num_train_users())
         self.data_size = data_size  # pyre-ignore
         return data_provider, data_loader.train_batch_size
 
@@ -133,8 +133,8 @@ class TestDifferentialPrivacyIntegration:
             train_batch_size,
         )
         model_wrapper.model.train()
-        for one_user_data in data_provider.train_data():
-            for batch in one_user_data:
+        for one_user_data in data_provider.train_users():
+            for batch in one_user_data.train_data():
                 batch_metrics = model_wrapper.fl_forward(batch)
                 loss = batch_metrics.loss
                 loss.backward()
@@ -225,7 +225,7 @@ class TestDifferentialPrivacyIntegration:
         global_fl_model, _eval_metric = sync_trainer.train(
             data_provider,
             metrics_reporter,
-            num_total_users=data_provider.num_users(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=world_size,
         )
 
@@ -317,7 +317,7 @@ class TestDifferentialPrivacyIntegration:
 
     def test_frameworks_one_client_sample_dp_off(self) -> None:
         """
-        Tests if Pytorch-DP and FL simulator, generate the same resutls, when
+        Tests if Pytorch-DP and FL simulator generate the same results when
         sample-level DP is off. Essentially, tests equivalence of the following
         scenarios, when there is 1 user:
 
@@ -325,7 +325,7 @@ class TestDifferentialPrivacyIntegration:
         2. Using Pytorch DP, when sample-level DP is off
 
         Note:
-            For this test, user-level DP is off, since it is not relavant to Pytorch-DP's
+            For this test, user-level DP is off, since it is not relevant to Pytorch-DP's
             provided functionality, which is sample-level DP.
         """
         lr = 1e-2
@@ -346,7 +346,7 @@ class TestDifferentialPrivacyIntegration:
 
     def test_frameworks_one_client_clipping_only(self) -> None:
         """
-        Tests if Pytorch-DP and FL simulator, generate the same resutls, when there
+        Tests if Pytorch-DP and FL simulator generate the same results when there
         is only 1 user and when sample-level DP is on (with noising off).
         Essentially, tests equivalence of the following scenarios:
 
@@ -355,7 +355,7 @@ class TestDifferentialPrivacyIntegration:
         (DP parameters are the same for the 2 scenarios.)
 
         Note:
-            For this test, user-level DP is off, since it is not relavant to Pytorch-DP's
+            For this test, user-level DP is off, since it is not relevant to Pytorch-DP's
             provided functionality, which is sample-level DP.
         """
         lr = 0.25
@@ -392,7 +392,6 @@ class TestDifferentialPrivacyIntegration:
         """
         Tests if user-level DP is equivalent to sample-level DP, when we
         have one user, and that user has one example (one epoch)
-
         """
         lr = 1.0
         momentum = 0.0
@@ -594,17 +593,17 @@ class TestDifferentialPrivacyIntegration:
             train_metrics_reported_per_epoch=10,
             eval_epoch_frequency=0.1,
         )
-        # we have 26 users, 4 users per round, that makes 7 rounds per epcoh
+        # we have 26 users, 4 users per round, that makes 7 rounds per epoch
         # we are reporting train, aggregation, and eval metrics all
         # at 10 rounds per epoch, so we should get a total of 21 reports
 
         def count_word(result, word):
             return str(result).count(word)
 
-        # check for existance of sample_dp and user_dp in reports.
-        # they are logged only in Aggrefation and we aggregated 7 times.
+        # Check for existence of sample_dp and user_dp in reports.
+        # They are logged only in Aggregation and we aggregated 7 times.
 
-        # print to std our prints sample_dp wich is a dict of four values once
+        # print to stdout: prints sample_dp, which is a dict of four values once
         # hence we should get 7 occurrences.
         assertEqual(
             count_word(metrics_reporter.stdout_results, "sample level dp"),
@@ -638,21 +637,21 @@ class TestDifferentialPrivacyIntegration:
         clip_norm,
         data_provider,
         buffer_size,
-        epochs=1,
+        epochs: int = 1,
     ):
         local_lr = np.random.sample()
         global_lr = np.random.sample()
 
         dp_model = DummyAlphabetFLModel()
-        nondp_model = copy.deepcopy(dp_model)
+        nondp_model = FLModelParamUtils.clone(dp_model)
         staleness_config = PolynomialStalenessWeightConfig(
             exponent=0.5, avg_staleness=0
         )
 
-        aggregator_config_nondp = FedAvgWithLRHybridAggregatorConfig(
+        aggregator_config_nondp = FedAvgWithLRFedBuffAggregatorConfig(
             lr=global_lr, buffer_size=buffer_size
         )
-        aggregator_config_dp = FedAvgWithLRHybridAggregatorConfig(
+        aggregator_config_dp = FedAvgWithLRFedBuffAggregatorConfig(
             lr=global_lr, buffer_size=buffer_size
         )
 
