@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -8,16 +9,16 @@ import pytest
 import torch
 from flsim.common.pytest_helper import (
     assertAlmostEqual,
-    assertTrue,
     assertEqual,
     assertFalse,
+    assertTrue,
 )
 from flsim.privacy.user_update_clip import UserUpdateClipper
-from flsim.tests import utils
+from flsim.utils import test_utils as utils
 
 
 @pytest.fixture(scope="class")
-def prepare_user_update_clipper_test(request):
+def prepare_user_update_clipper_test(request) -> None:
     request.cls.user_update_clipper = UserUpdateClipper()
 
     def _init_user_model(param_value):
@@ -44,10 +45,12 @@ class TestUserUpdateClipper:
         self.user_update_clipper.clip(clipped_model_diff, max_norm)
         return clipped_model_diff
 
-    def test_calc_clip_factor(self):
+    def test_calc_clip_factor(self) -> None:
         """
         Tests that the clip factor for user updates is calculated correctly.
         """
+        # pyre-fixme[16]: `TestUserUpdateClipper` has no attribute
+        #  `user_update_clipper`.
         clip_factor = self.user_update_clipper._calc_clip_factor(
             max_norm=5, per_user_norm=10
         )
@@ -63,7 +66,7 @@ class TestUserUpdateClipper:
         )
         assertAlmostEqual(clip_factor, 1, places=3)
 
-    def test_calc_user_update_norm(self):
+    def test_calc_user_update_norm(self) -> None:
         """
         Tests that the user update l-2 norms are calculated correctly.
         """
@@ -71,6 +74,8 @@ class TestUserUpdateClipper:
         model.fill_all(2.0)
         model_params = model.parameters()
         # norm = sqrt(21*2^2)=sqrt(84)=9.16515138991168
+        # pyre-fixme[16]: `TestUserUpdateClipper` has no attribute
+        #  `user_update_clipper`.
         norm = self.user_update_clipper._calc_norm(model_params)
         assertTrue(torch.allclose(norm, torch.tensor(9.16515138991168), rtol=1e-06))
 
@@ -80,13 +85,15 @@ class TestUserUpdateClipper:
         norm = self.user_update_clipper._calc_norm(model_params)
         assertTrue(torch.allclose(norm, torch.tensor(4.58257569495584), rtol=1e-06))
 
-    def test_clipped_updates_are_smaller(self):
+    def test_clipped_updates_are_smaller(self) -> None:
         """
         Tests that user updates are clipped and their value is smaller than
         the original updates
         """
 
         # assert the parameters of model_diff are all = (7 - 6 = 1)
+        # pyre-fixme[16]: `TestUserUpdateClipper` has no attribute
+        #  `original_model_diff_params`.
         for p in self.original_model_diff_params:
             assertTrue(torch.allclose(p.float(), torch.tensor(1.0)))
 
@@ -99,7 +106,7 @@ class TestUserUpdateClipper:
         ):
             assertTrue(torch.all(original.gt(clipped)))
 
-    def test_clipped_user_updates_non_zero(self):
+    def test_clipped_user_updates_non_zero(self) -> None:
         """
         Tests that user updates are not zero by clipping
         """
@@ -111,13 +118,16 @@ class TestUserUpdateClipper:
             allzeros = torch.zeros_like(clipped)
             assertFalse(torch.allclose(clipped, allzeros))
 
-    def test_clipping_to_high_value_does_not_clip(self):
+    def test_clipping_to_high_value_does_not_clip(self) -> None:
         """
         Tests that when clip value is set too high, user
         updates are not clipped
         """
         clipped_model_diff = self._init_clipped_model_diff(9999)
         mismatched = utils.verify_models_equivalent_after_training(
-            self.original_model_diff, clipped_model_diff
+            # pyre-fixme[16]: `TestUserUpdateClipper` has no attribute
+            #  `original_model_diff`.
+            self.original_model_diff,
+            clipped_model_diff,
         )
         assertEqual(mismatched, "")
