@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -13,8 +14,7 @@ from typing import Optional
 from flsim.channels.base_channel import IdentityChannel
 from flsim.interfaces.model import IFLModel
 from flsim.reducers.dp_round_reducer import DPRoundReducer, DPRoundReducerConfig
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
+from flsim.utils.config_utils import fullclassname, init_self_cfg
 from flsim.utils.fl.common import FLModelParamUtils
 from omegaconf import MISSING
 from torch import nn
@@ -135,7 +135,7 @@ class WeightedDPRoundReducer(DPRoundReducer):
 
     def _unbiased_estimator(self) -> float:
         """
-        For weighted averaged reductions, the unbiased estimator calculates the true
+        For weighted average reductions, the unbiased estimator calculates the true
         weighted average of the models and the sensitivity of it will be:
             (clipping_value * max_weight) / (min_weight * users_per_round)
         """
@@ -182,7 +182,7 @@ class WeightedDPRoundReducerConfig(DPRoundReducerConfig):
     Allowing weights in dp should generally be avoided unless weights
     are in the same range. If weights are extremely different, one
     might as well throw updates from clients with smaller weights
-    away as they will be drawned in noise.
+    away as they will be drowned in noise.
     """
     _target_: str = fullclassname(WeightedDPRoundReducer)
     min_weight: float = 1e-6
