@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -9,7 +10,7 @@ from __future__ import annotations
 import logging
 import math
 from dataclasses import dataclass
-from typing import Any, List, Tuple, Optional
+from typing import Any, List, Optional, Tuple
 
 from flsim.clients.async_client import AsyncClientDevice
 from flsim.common.logger import Logger
@@ -20,10 +21,10 @@ from flsim.data.data_provider import IFLDataProvider
 from flsim.interfaces.metrics_reporter import IFLMetricsReporter, Metric
 from flsim.interfaces.model import IFLModel
 from flsim.optimizers.async_aggregators import (
+    AsyncAggregator,
     AsyncAggregatorConfig,
     FedAvgWithLRAsyncAggregatorConfig,
-    AsyncAggregator,
-    HybridAggregator,
+    FedBuffAggregator,
 )
 from flsim.trainers.trainer_base import FLTrainer, FLTrainerConfig
 from flsim.utils.async_trainer.async_user_selector import (
@@ -35,9 +36,8 @@ from flsim.utils.async_trainer.training_event_generator import (
     AsyncTrainingEventGeneratorConfig,
     EventGeneratorConfig,
 )
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
-from flsim.utils.cuda import GPUMemoryMinimizer, CudaTransferMinimizer
+from flsim.utils.config_utils import fullclassname, init_self_cfg
+from flsim.utils.cuda import CudaTransferMinimizer, GPUMemoryMinimizer
 from hydra.utils import instantiate
 from omegaconf import OmegaConf
 from tqdm import tqdm
@@ -88,9 +88,10 @@ class AsyncTrainer(FLTrainer, AsyncTrainingEventHandler):
         self._event_generator = instantiate(
             self.cfg.training_event_generator,
         )
-        # for pyre; declare instance variables (https://fburl.com/88n6i71r)
+        # for pyre; declare instance variables
+        # https://stackoverflow.com/questions/56584140/python-3-5-type-annotated-variable-without-initial-value
         # pyre-fixme[8]: Attribute has type `IFLMetricsReporter`; used as `None`.
-        self.metric_reporter = None  # type: IFLMetricsReporter
+        self.metrics_reporter: "IFLMetricsReporter" = None
         self.best_metric = None
         self.best_model_state = self.global_model().fl_get_module().state_dict()
 
@@ -137,7 +138,7 @@ class AsyncTrainer(FLTrainer, AsyncTrainingEventHandler):
         self.aggregator.zero_grad()
         # 2. train client on local data
         client_delta, final_local_model, num_examples = client.train_local_model(
-            self.metric_reporter
+            self.metrics_reporter
         )
         assert num_examples > 0, "Client must have more than one example"
         # 3. get client staleness
@@ -163,7 +164,7 @@ class AsyncTrainer(FLTrainer, AsyncTrainingEventHandler):
         return staleness <= self.cfg.max_staleness
 
     def _num_global_steps_in_epoch(self):
-        if isinstance(self.aggregator, HybridAggregator):
+        if isinstance(self.aggregator, FedBuffAggregator):
             return math.ceil(self.num_total_users / self.aggregator.cfg.buffer_size)
         else:
             return self.num_total_users
@@ -173,20 +174,19 @@ class AsyncTrainer(FLTrainer, AsyncTrainingEventHandler):
         self._report_train_metrics(
             model=self.global_model(),
             timeline=timeline,
-            metric_reporter=self.metric_reporter,
+            metrics_reporter=self.metrics_reporter,
             extra_metrics=self._get_training_metrics(),
         )
         self._calc_post_epoch_communication_metrics(
-            self._get_timeline(), self.metric_reporter
+            self._get_timeline(), self.metrics_reporter
         )
-        (self.best_metric, self.best_model_state,) = FLTrainer._maybe_run_evaluation(
+        self.best_metric, self.best_model_state = FLTrainer._maybe_run_evaluation(
             self,
-            self.global_model(),
-            timeline,
-            self.data_provider.eval_data(),
-            self.metric_reporter,
-            self.best_metric,
-            self.best_model_state,
+            timeline=timeline,
+            data_provider=self.data_provider,
+            metrics_reporter=self.metrics_reporter,
+            best_metric=self.best_metric,
+            best_model_state=self.best_model_state,
         )
 
     def _get_training_metrics(self) -> List[Metric]:
@@ -221,7 +221,7 @@ class AsyncTrainer(FLTrainer, AsyncTrainingEventHandler):
     def train(
         self,
         data_provider: IFLDataProvider,
-        metric_reporter: IFLMetricsReporter,
+        metrics_reporter: IFLMetricsReporter,
         num_total_users: int,
         distributed_world_size: int = 1,
         rank: int = 0,
@@ -232,7 +232,7 @@ class AsyncTrainer(FLTrainer, AsyncTrainingEventHandler):
                 each batch iterator represents data from a single 'user'
             eval_iter (Iterable[Any]): batch iterator of evaluation data
             model (Model): model to be trained
-            metric_reporter (IFLMetricsReporter): compute metric based on training
+            metrics_reporter (IFLMetricsReporter): compute metric based on training
                 output and report results to console, file.. etc
 
         Returns:
@@ -245,8 +245,8 @@ class AsyncTrainer(FLTrainer, AsyncTrainingEventHandler):
         self.best_metric = None
         self.best_model_state = self.global_model().fl_get_module().state_dict()
         self.data_provider = data_provider
-        self.metric_reporter = metric_reporter
-        self.num_total_users = data_provider.num_users()
+        self.metrics_reporter = metrics_reporter
+        self.num_total_users = data_provider.num_train_users()
         self.aggregator.set_num_total_users(self.num_total_users)
         user_selector = AsyncUserSelectorFactory.create_users_selector(
             # pyre-fixme[16]: `AsyncTrainer` has no attribute `cfg`.
