@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -11,14 +12,13 @@ from typing import Any, Iterable, List, Optional, Tuple
 
 import torch
 from flsim.clients.base_client import Client
-from flsim.clients.dp_client import DPClientConfig, DPClient
+from flsim.clients.dp_client import DPClient, DPClientConfig
 from flsim.data.data_provider import IFLDataProvider
 from flsim.interfaces.metrics_reporter import IFLMetricsReporter, Metric
 from flsim.interfaces.model import IFLModel
 from flsim.reducers.dp_round_reducer import DPRoundReducerConfig
 from flsim.trainers.sync_trainer import SyncTrainer, SyncTrainerConfig, Timeline
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
+from flsim.utils.config_utils import fullclassname, init_self_cfg
 from hydra.utils import instantiate
 from omegaconf import OmegaConf
 
@@ -54,7 +54,7 @@ class PrivateSyncTrainer(SyncTrainer):
     def train(
         self,
         data_provider: IFLDataProvider,
-        metric_reporter: IFLMetricsReporter,
+        metrics_reporter: IFLMetricsReporter,
         num_total_users: int,
         distributed_world_size: int,
         rank: int = 0,
@@ -71,7 +71,7 @@ class PrivateSyncTrainer(SyncTrainer):
         self.aggregator.init_round(dp_round_reducer)
         return super().train(
             data_provider,
-            metric_reporter,
+            metrics_reporter,
             num_total_users,
             distributed_world_size,
             rank,
@@ -98,13 +98,10 @@ class PrivateSyncTrainer(SyncTrainer):
         self,
         clients: Iterable[Client],
         model: IFLModel,
-        timeline: Timeline,
-        metric_reporter: Optional[IFLMetricsReporter],
+        metrics_reporter: Optional[IFLMetricsReporter],
     ) -> List[Metric]:
 
-        metrics = super().calc_post_aggregation_train_metrics(
-            clients, model, timeline, metric_reporter
-        )
+        metrics = super()._calc_privacy_metrics(clients, model, metrics_reporter)
 
         # calculate sample level dp privacy loss statistics.
         all_client_eps = torch.Tensor(
