@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -10,19 +11,18 @@ import abc
 import logging
 import sys
 from dataclasses import dataclass
-from typing import Any, Iterable, List, Optional, Tuple
+from typing import Any, List, Optional, Tuple
 
 import torch
 from flsim.channels.base_channel import FLChannelConfig
-from flsim.channels.communication_stats import (
-    ChannelDirection,
-)
+from flsim.channels.communication_stats import ChannelDirection
 from flsim.clients.base_client import ClientConfig
+from flsim.common.fine_tuner import FineTuner
 from flsim.common.logger import Logger
 from flsim.common.timeline import Timeline
 from flsim.common.timeout_simulator import (
-    TimeOutSimulatorConfig,
     NeverTimeOutSimulatorConfig,
+    TimeOutSimulatorConfig,
 )
 from flsim.data.data_provider import IFLDataProvider
 from flsim.interfaces.metrics_reporter import IFLMetricsReporter, Metric, TrainingStage
@@ -30,12 +30,11 @@ from flsim.interfaces.model import IFLModel
 from flsim.utils.config_utils import init_self_cfg
 from flsim.utils.cuda import CudaTransferMinimizer
 from hydra.utils import instantiate
-from omegaconf import MISSING
-from omegaconf import OmegaConf
+from omegaconf import MISSING, OmegaConf
 
 
 class FLTrainer(abc.ABC):
-    """Base class for FederatedLearning Training"""
+    """Base class for Federated Learning Training"""
 
     logger: logging.Logger = Logger.get_logger(__name__)
 
@@ -71,9 +70,14 @@ class FLTrainer(abc.ABC):
         self.num_total_users: int = -1
         # Initialize tracker for measuring communication between the clients and
         # the server if communication metrics are enabled
+        self.clients = {}
+        self.eval_clients = {}
 
     @classmethod
     def _set_defaults_in_cfg(cls, cfg):
+        """Set default config if missing.
+        Sub-classes may further set default for other components.
+        """
         if OmegaConf.is_missing(cfg.timeout_simulator, "_target_"):
             cfg.timeout_simulator = NeverTimeOutSimulatorConfig()
         if OmegaConf.is_missing(cfg.client, "_target_"):
@@ -85,7 +89,7 @@ class FLTrainer(abc.ABC):
     def train(
         self,
         data_provider: IFLDataProvider,
-        metric_reporter: IFLMetricsReporter,
+        metrics_reporter: IFLMetricsReporter,
         num_total_users: int,
         distributed_world_size: int = 1,
         rank: int = 0,
@@ -93,13 +97,13 @@ class FLTrainer(abc.ABC):
         pass
 
     def test(
-        self, data_iter: Iterable[Any], metric_reporter: IFLMetricsReporter
+        self, data_provider: IFLDataProvider, metrics_reporter: IFLMetricsReporter
     ) -> Any:
         return self._test(
             timeline=Timeline(global_round=1),
-            data_iter=data_iter,
+            data_provider=data_provider,
             model=self.global_model(),
-            metric_reporter=metric_reporter,
+            metrics_reporter=metrics_reporter,
         )
 
     def global_model(self) -> IFLModel:
@@ -107,35 +111,45 @@ class FLTrainer(abc.ABC):
 
     def _maybe_run_evaluation(
         self,
-        model: IFLModel,
         timeline: Timeline,
-        eval_iter: Iterable[Any],
-        metric_reporter: IFLMetricsReporter,
+        data_provider,
+        metrics_reporter: IFLMetricsReporter,
         best_metric,
         best_model_state,
     ):
+
         # pyre-fixme[16]: `FLTrainer` has no attribute `cfg`.
+        # Skip evaluation
         if not self.cfg.do_eval:
             return best_metric, best_model_state
         if not timeline.tick(self.cfg.eval_epoch_frequency):
             return best_metric, best_model_state
+
+        personalized_metrics = {}
+        if self.cfg.personalized:
+            # Personalized eval on eval users (i.e. finetune for each eval user first)
+            # TODO: Fix metrics reporting in order to report personalized metrics
+            personalized_metrics = self._evaluate_personalized_eval_users(  # noqa
+                timeline=timeline,
+                data_provider=data_provider,
+                global_model=self.global_model(),
+                metrics_reporter=metrics_reporter,
+            )
+
+        # Evaluate global model on eval users
         eval_metric, eval_metric_better_than_prev = self._evaluate(
             timeline=timeline,
-            data_iter=eval_iter,
-            model=model,
-            metric_reporter=metric_reporter,
+            data_provider=data_provider,
+            global_model=self.global_model(),
+            metrics_reporter=metrics_reporter,
         )
 
-        # 1) keep the best model so far if metric_reporter.compare_metrics is specified
-        # 2) if self.always_keep_trained_model is set as true, ignore the metricsa and
-        #    keep the trained model for each epoch
+        # 1) Keep the best model so far if metrics_reporter.compare_metrics is specified.
+        # 2) If self.always_keep_trained_model is True, ignore the metrics and keep the
+        #    trained model for each epoch.
         if self.cfg.always_keep_trained_model or eval_metric_better_than_prev:
-            # last_best_epoch = epoch
-            self.logger.info(
-                f"Found a better model!, current_eval_metric:{eval_metric}"
-            )
             best_metric = eval_metric
-            model_state = model.fl_get_module().state_dict()
+            model_state = self.global_model().fl_get_module().state_dict()
             best_model_state = model_state
         sys.stdout.flush()
         return best_metric, best_model_state
@@ -147,17 +161,26 @@ class FLTrainer(abc.ABC):
         self,
         model: IFLModel,
         timeline: Timeline,
-        metric_reporter: Optional[IFLMetricsReporter] = None,
+        metrics_reporter: Optional[IFLMetricsReporter] = None,
         extra_metrics: Optional[List[Metric]] = None,
     ) -> None:
+        """Reports train metrics (e.g. loss and accuracy) after one round.
+        Args:
+            model: Model to calculate train metrics. Can either be global or client-side
+                model.
+            timeline: Timeline object that keeps track of current point of time, such as
+                current round and epoch.
+            metrics_reporter: Metrics reporter object. If none, do not report metrics.
+            extra_metrics: Miscellaneous metrics in addition to train loss and scores.
+        """
         if (
             # pyre-fixme[16]: `FLTrainer` has no attribute `cfg`.
             self.cfg.report_train_metrics
-            and metric_reporter is not None
+            and metrics_reporter is not None
             and timeline.tick(1.0 / self.cfg.train_metrics_reported_per_epoch)
         ):
             self._print_training_stats(timeline)
-            metric_reporter.report_metrics(
+            metrics_reporter.report_metrics(
                 model=model,
                 reset=True,
                 stage=TrainingStage.TRAINING,
@@ -168,11 +191,15 @@ class FLTrainer(abc.ABC):
             )
 
     def _calc_post_epoch_communication_metrics(
-        self, timeline: Timeline, metric_reporter: Optional[IFLMetricsReporter]
+        self, timeline: Timeline, metrics_reporter: Optional[IFLMetricsReporter]
     ):
+        """Calculates communication metrics after an epoch ends, such as amount of data
+        communicated bewteen server and client.
+        TODO: This should really be called `post_round` instead of `post_epoch`.
+        """
 
         if (
-            metric_reporter is not None
+            metrics_reporter is not None
             and self.channel.cfg.report_communication_metrics
             # pyre-fixme[16]: `FLTrainer` has no attribute `cfg`
             and timeline.tick(1.0 / self.cfg.train_metrics_reported_per_epoch)
@@ -186,7 +213,7 @@ class FLTrainer(abc.ABC):
                 )
                 for name, tracker in self.channel.stats_collector.get_channel_stats().items()
             ]
-            metric_reporter.report_metrics(
+            metrics_reporter.report_metrics(
                 model=None,
                 reset=False,
                 stage=TrainingStage.TRAINING,
@@ -197,50 +224,96 @@ class FLTrainer(abc.ABC):
             )
             self.channel.stats_collector.reset_channel_stats()
 
+    def _evaluate_personalized_eval_users(
+        self,
+        timeline: Timeline,
+        data_provider,
+        global_model: IFLModel,  # a global model
+        metrics_reporter: IFLMetricsReporter,
+    ) -> Any:
+        """Perform personalized evaluation on evaluation users.
+        Specifically, finetune global model for each eval user on their training data
+        first and then evaluate performance on eval users' local eval data.
+        """
+        print(
+            f"{timeline}: \t Evaluate global model w/ finetune on validation data of eval users"
+        )
+        personalized_metrics, _ = FineTuner.fine_tune_and_evaluate(
+            data=self.data_provider.eval_users(),
+            global_model=global_model,
+            # pyre-ignore[16]
+            client_config=self.cfg.client,
+            metrics_reporter=metrics_reporter,
+            cuda_state_manager=self._cuda_state_manager,
+            training_stage=TrainingStage.PERSONALIZED_EVAL,
+            timeline=timeline,
+            epochs=self.cfg.personalized_epochs,
+        )
+        return personalized_metrics
+
     def _evaluate(
         self,
         timeline: Timeline,
-        data_iter: Iterable[Any],
-        model: IFLModel,
-        metric_reporter: IFLMetricsReporter,
+        data_provider: IFLDataProvider,
+        global_model: IFLModel,
+        metrics_reporter: IFLMetricsReporter,
     ) -> Tuple[Any, bool]:
+        """
+        Evaluate global model on eval users
+        """
         with torch.no_grad():
-            self._cuda_state_manager.before_train_or_eval(model)
-            model.fl_get_module().eval()
-            print(f"Running {timeline} for {TrainingStage.EVAL.name.title()}")
+            self._cuda_state_manager.before_train_or_eval(global_model)
+            global_model.fl_get_module().eval()
+            print(f"{timeline}: \t Evaluates global model on all data of eval users")
+            for user in data_provider.eval_users():
+                for batch in user.eval_data():
+                    batch_metrics = global_model.get_eval_metrics(batch)
+                    metrics_reporter.add_batch_metrics(batch_metrics)
 
-            for _, batch in enumerate(data_iter):
-                batch_metrics = model.get_eval_metrics(batch)
-                metric_reporter.add_batch_metrics(batch_metrics)
-
-            metrics, found_best_model = metric_reporter.report_metrics(
-                model=model,
+            metrics, found_best_model = metrics_reporter.report_metrics(
+                model=global_model,
                 reset=True,
                 stage=TrainingStage.EVAL,
                 timeline=timeline,
                 epoch=timeline.global_round_num(),  # for legacy
                 print_to_channels=True,
             )
-            self._cuda_state_manager.after_train_or_eval(model)
+            self._cuda_state_manager.after_train_or_eval(global_model)
             return metrics, found_best_model
 
     def _test(
         self,
         timeline: Timeline,
-        data_iter: Iterable[Any],
+        data_provider: IFLDataProvider,
         model: IFLModel,
-        metric_reporter: IFLMetricsReporter,
+        metrics_reporter: IFLMetricsReporter,
     ) -> Any:
+
+        personalized_metrics = {}
+        # pyre-ignore[16]
+        if self.cfg.personalized:
+            personalized_metrics, _ = FineTuner.fine_tune_and_evaluate(  # noqa
+                data=self.data_provider.test_users(),
+                global_model=model,
+                client_config=self.cfg.client,
+                metrics_reporter=metrics_reporter,
+                cuda_state_manager=self._cuda_state_manager,
+                training_stage=TrainingStage.PERSONALIZED_TEST,
+                timeline=timeline,
+                epochs=self.cfg.personalized_epochs,
+            )
+
         with torch.no_grad():
             self._cuda_state_manager.before_train_or_eval(model)
             model.fl_get_module().eval()
             print(f"Running {timeline} for {TrainingStage.TEST.name.title()}")
 
-            for _, batch in enumerate(data_iter):
-                batch_metrics = model.get_eval_metrics(batch)
-                metric_reporter.add_batch_metrics(batch_metrics)
+            for test_user in data_provider.test_users():
+                for batch in test_user.eval_data():
+                    batch_metrics = model.get_eval_metrics(batch)
+                    metrics_reporter.add_batch_metrics(batch_metrics)
 
-            metrics, _ = metric_reporter.report_metrics(
+            metrics, _ = metrics_reporter.report_metrics(
                 model=model,
                 reset=True,
                 stage=TrainingStage.TEST,
@@ -257,7 +330,7 @@ class FLTrainerConfig:
     _target_: str = MISSING
     _recursive_: bool = False
     # Training epochs
-    epochs: float = 10.0
+    epochs: float = 1000.0
     # Whether to do evaluation and model selection based on it.
     do_eval: bool = True
     # don't use metric reporter to choose: always keep trained model
@@ -277,3 +350,5 @@ class FLTrainerConfig:
     client: ClientConfig = ClientConfig()
     # config for the channels
     channel: FLChannelConfig = FLChannelConfig()
+    personalized: bool = False  # flag to personalized global model by locally fine tuning before evaluation
+    personalized_epochs: int = 1  # number of fine tune epochs to run
