@@ -1,12 +1,13 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
 import abc
 from dataclasses import dataclass
-from enum import Enum, auto
+from enum import auto, Enum
 
 import numpy as np
 from flsim.data.data_provider import IFLDataProvider, IFLUserData
@@ -18,7 +19,7 @@ class AsyncUserSelectorInfo:
     Dataclass to encapsulate a selected user for async training
 
     user_data (IFLUserData): seleected user data in the dataset
-    user_index (int): the index for user_data assuming IFLDataProvider.train_data is a List
+    user_index (int): the index for user_data assuming IFLDataProvider.train_users is a List
     """
     user_data: IFLUserData
     user_index: int
@@ -41,8 +42,11 @@ class RandomAsyncUserSelector(AsyncUserSelector):
         super().__init__(data_provider)
 
     def get_random_user(self) -> AsyncUserSelectorInfo:
-        user_index = np.random.randint(0, self.data_provider.num_users())
-        return AsyncUserSelectorInfo(self.data_provider[user_index], user_index)
+        user_index = np.random.randint(0, self.data_provider.num_train_users())
+        return AsyncUserSelectorInfo(
+            user_data=self.data_provider.get_train_user(user_index),
+            user_index=user_index,
+        )
 
 
 class RoundRobinAsyncUserSelector(AsyncUserSelector):
@@ -59,8 +63,11 @@ class RoundRobinAsyncUserSelector(AsyncUserSelector):
         user_index = self.current_user_index
         self.current_user_index = (
             self.current_user_index + 1
-        ) % self.data_provider.num_users()
-        return AsyncUserSelectorInfo(self.data_provider[user_index], user_index)
+        ) % self.data_provider.num_train_users()
+        return AsyncUserSelectorInfo(
+            user_data=self.data_provider.get_train_user(user_index),
+            user_index=user_index,
+        )
 
 
 class AsyncUserSelectorType(Enum):
