@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -10,22 +11,20 @@ class attributes for our Configs and Pyre just complains that it cannot
 figure out where the attributes are getting initialized. Putting Optional here
 is not a choice for us, because we want to differentiate between Optional and
 strictly required attributes in our Config classes.
-Reference: https://fburl.com/4cdf3akr
 """
 from __future__ import annotations
 
 from dataclasses import dataclass
 
 from flsim.utils.async_trainer.async_example_weights import (
-    EqualExampleWeightConfig,
     AsyncExampleWeightConfig,
+    EqualExampleWeightConfig,
 )
 from flsim.utils.async_trainer.async_staleness_weights import (
     AsyncStalenessWeightConfig,
     ConstantStalenessWeightConfig,
 )
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
+from flsim.utils.config_utils import fullclassname, init_self_cfg
 from flsim.utils.fl.stats import RandomVariableStatsTracker
 from hydra.utils import instantiate
 from omegaconf import OmegaConf
