@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -12,6 +13,7 @@ from flsim.interfaces.model import IFLModel
 
 def FloatTensor(cuda_enabled: bool, *args):
     if cuda_enabled:
+        # pyre-fixme[16]: Module `cuda` has no attribute `FloatTensor`.
         return torch.cuda.FloatTensor(*args)
     else:
         return torch.FloatTensor(*args)
@@ -21,7 +23,7 @@ def tensor(data, dtype, cuda_enabled: bool):
     return torch.tensor(data, dtype=dtype, device=device(cuda_enabled))
 
 
-def device(cuda_enabled: bool):
+def device(cuda_enabled: bool) -> str:
     return "cuda:{}".format(torch.cuda.current_device()) if cuda_enabled else "cpu"
 
 
