@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -7,7 +8,8 @@
 import logging
 from enum import IntEnum
 from itertools import chain
-from typing import List, Iterable
+from typing import Iterable, List, Optional, Tuple
+from warnings import warn
 
 import torch
 import torch.distributed as dist
@@ -25,10 +27,9 @@ class OperationType(IntEnum):
 
 class FLDistributedUtils:
     """
-    More detailed note: https://fburl.com/5n8mnaf3. We cannot use PyTorch
-    DDP here, because DDP is tied to backward() and only provides
-    high-level APIs for reducing gradients, but in FL, after each round, we
-    need to perform all-reduce on the models (optionally with some pre and
+    We cannot use PyTorch DDP here, because DDP is tied to backward() and only
+    provides high-level APIs for reducing gradients, but in FL, after each round,
+    we need to perform all-reduce on the models (optionally with some pre and
     post-processing) instead of just model gradients (e.g. FedAvg).
     We reduce the number of all-reduce operations by flatten an entire model
     into a 1D tensor, if the model size is less than a buffer limit of 256MB.
@@ -39,16 +40,17 @@ class FLDistributedUtils:
     logger: logging.Logger = Logger.get_logger(__name__)
 
     # equivalent to 256 MB of floats, same buffer size as in PyTorch DDP
-    MAX_BUFFER_SIZE = 2 ** 28
-    WORLD_SIZE = 1
-    # allow distributed training on CPU, default False
+    MAX_BUFFER_SIZE = 2**28
+    WORLD_SIZE = 1  # number of processes
+    NUM_WORKERS = 1  # number of CPUs or GPUs
+    # run distributed training on CPU, default False
     DISTRIBUTED_TRAINING_ON_CPU = False
-    DISTRIBUTED_BACKED = dist.Backend.NCCL
+    DISTRIBUTED_BACKEND = dist.Backend.NCCL
 
     @classmethod
     def distributed_training_on_cpu(cls):
         cls.DISTRIBUTED_TRAINING_ON_CPU = True
-        cls.DISTRIBUTED_BACKED = dist.Backend.GLOO
+        cls.DISTRIBUTED_BACKEND = dist.Backend.GLOO
 
     @classmethod
     def distributed_training_on_cuda(cls):
@@ -58,34 +60,55 @@ class FLDistributedUtils:
         cpu distributed again.
         """
         cls.DISTRIBUTED_TRAINING_ON_CPU = False
-        cls.DISTRIBUTED_BACKED = dist.Backend.NCCL
+        cls.DISTRIBUTED_BACKEND = (
+            dist.Backend.GLOO
+            if (cls.WORLD_SIZE > cls.NUM_WORKERS or cls.NUM_WORKERS % cls.WORLD_SIZE)
+            else dist.Backend.NCCL
+        )
 
     @classmethod
-    def check_distributed_training_configs(cls, distributed_world_size: int):
-        if cls.DISTRIBUTED_TRAINING_ON_CPU:
-            assert distributed_world_size <= mp.cpu_count(), (
-                f"Only {mp.cpu_count()} GPUs are available, "
-                f"but {distributed_world_size} CPUs were requested."
-            )
-        elif distributed_world_size > 1:
+    def calc_num_processes_and_workers(
+        cls, distributed_world_size: int, cuda: bool
+    ) -> Tuple[int, int]:
+        """
+        Checks resources on the machine and returns
+        the distributed world size and the number of workers.
+
+        For cpu we do not allow more than one process per cpu.
+        For cuda we do
+        """
+        if cuda:
             assert torch.cuda.is_available(), (
                 "distributed_world_size is greater than 1 "
-                "use only if cuda is supported or distributed_training_on_cpu"
+                "use only if cuda is supported or distributed_training_on_cuda"
                 "has been called!"
             )
-            assert distributed_world_size <= torch.cuda.device_count(), (
-                f"Only {torch.cuda.device_count()} GPUs are available, "
-                f"but {distributed_world_size} GPUs were requested."
-            )
+            num_gpus = torch.cuda.device_count()
+            if distributed_world_size > num_gpus and distributed_world_size % num_gpus:
+                warn(
+                    f"There are {num_gpus} physical cuda workers (i.e gpus), "
+                    f"you are asking {distributed_world_size} workers, "
+                    "we need equal number of workers per gpu"
+                )
+            return distributed_world_size, num_gpus
+        else:
+            num_cpus = mp.cpu_count()
+            if distributed_world_size > num_cpus:
+                raise Warning(
+                    f"Only {num_cpus} CPUs are available, "
+                    f"but {distributed_world_size} workers were requested."
+                )
+            return min(distributed_world_size, num_cpus), num_cpus
 
     @classmethod
     def setup_distributed_training(cls, distributed_world_size: int, use_cuda=True):
-        cls.WORLD_SIZE = distributed_world_size
+        cls.WORLD_SIZE, cls.NUM_WORKERS = cls.calc_num_processes_and_workers(
+            distributed_world_size, use_cuda
+        )
         if use_cuda:
             cls.distributed_training_on_cuda()
         else:
             cls.distributed_training_on_cpu()
-        cls.check_distributed_training_configs(distributed_world_size)
 
     @classmethod
     def distributed_operation(
@@ -111,7 +134,7 @@ class FLDistributedUtils:
           next param in the list.
 
         At the end the function joins all async ops and puts processed values from each flattened
-        buffer in to their respective param.
+        buffer into their respective param.
 
         Note:
             In all operations it is assumed that the master worker is the worker with rank 0.
@@ -176,7 +199,7 @@ class FLDistributedUtils:
         'params', which is a list of tensors, and copy each tensor (which is
         avset of parameters from model) to buffer one by one. After that, we
         callvall_reduce() function in PyTorch distributed as an async
-        operation tovall processes in the group (and get async handle to
+        operation to all processes in the group (and get async handle to
         return after this).
 
         Args:
@@ -249,15 +272,28 @@ class FLDistributedUtils:
             if dst < 0:
                 cls.logger.debug("dst is not defined setting 0 as the default value")
                 dst = 0
-            cls.logger.warning("Operation reduce is not supported on CPU on ")
+            cls.logger.warning("Operation reduce is not supported on CPU.")
 
-            handle = dist.reduce(
-                buffer,
-                dst,
-                op=dist.ReduceOp.SUM,
-                group=cls._get_default_group(),
-                async_op=True,
-            )
+            if not (
+                cls.DISTRIBUTED_TRAINING_ON_CPU
+                or cls.DISTRIBUTED_BACKEND == dist.Backend.NCCL
+            ):
+                # GLOO on GPU does not support reduce
+                cls.logger.warning("Changing reduce operation to reduce all.")
+                handle = dist.all_reduce(
+                    buffer,
+                    op=dist.ReduceOp.SUM,
+                    group=cls._get_default_group(),
+                    async_op=True,
+                )
+            else:
+                handle = dist.reduce(
+                    buffer,
+                    dst,
+                    op=dist.ReduceOp.SUM,
+                    group=cls._get_default_group(),
+                    async_op=True,
+                )
         elif op == OperationType.BROADCAST:
             if src < 0:
                 cls.logger.debug(
@@ -283,8 +319,9 @@ class FLDistributedUtils:
         i.e. Copies all-reduced grads back into their original place. However,
         more generally speaking, what this function actually does is treating the
         'buffer' (i.e. the 2nd param) as a well-flattened 1D tensor of the list
-        of params and copy all the params back to buffer.
+        of params and copy all the params back to the buffer.
         """
+        # TODO: (jesikmin) T55869097 Check whether the size of buffer is same as
         # the total number of elements of params
         # copy all-reduced grads back into their original place
         offset = 0
@@ -300,7 +337,7 @@ class FLDistributedUtils:
     @classmethod
     def is_master_worker(cls):
         """
-        we assume that worker 0 is the master worker."
+        We assume that worker 0 is the master worker.
         """
         return (not dist.is_initialized()) or dist.get_rank() == 0
 
@@ -327,27 +364,30 @@ class FLDistributedUtils:
     ):
         cls.setup_distributed_training(world_size, use_cuda)
         if not cls.DISTRIBUTED_TRAINING_ON_CPU:
-            device = torch.device(f"cuda:{rank}")
+            device = torch.device(f"cuda:{rank % cls.NUM_WORKERS}")
             torch.cuda.set_device(device)
         if world_size > 1:
             dist.init_process_group(
-                backend=cls.DISTRIBUTED_BACKED,
+                backend=cls.DISTRIBUTED_BACKEND,
                 init_method=init_method,
                 world_size=world_size,
                 rank=rank,
             )
 
     @classmethod
-    def synchronize_across_ranks(
+    def synchronize_model_across_workers(
         cls,
-        model: nn.Module,
-        weights: torch.Tensor,
         operation: OperationType,
+        model: nn.Module,
+        weights: Optional[torch.Tensor] = None,
         only_federated_params: bool = False,
     ):
         state_dict = FLModelParamUtils.get_state_dict(
             model, only_federated_params=only_federated_params
         )
-        cls.distributed_operation(
-            params=chain([weights], state_dict.values()), op=operation
-        )
+        if weights is not None:
+            cls.distributed_operation(
+                params=chain([weights], state_dict.values()), op=operation
+            )
+        else:
+            cls.distributed_operation(params=state_dict.values(), op=operation)
