@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -34,7 +35,7 @@ class FLModelWithPrivateModules(abc.ABC):
 
     @classmethod
     def get_user_private_parameters(cls) -> Iterable[torch.Tensor]:
-        """Return emulated mapping that maps each user to her private params."""
+        """Return emulated mapping that maps each user to their private params."""
         for module in cls.user_private_module_dict.values():
             for param in module.parameters():
                 yield param
