@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -11,7 +12,7 @@ from flsim.utils.async_trainer.async_example_weights import (
     AsyncExampleWeightConfig,
     ExampleWeight,
 )
-from flsim.utils.tests.helpers.async_weights_test_utils import (  # noqa
+from flsim.utils.tests.helpers.test_async_weights_utils import (  # noqa
     AsyncExampleWeightsTestUtils,
 )
 from hydra.utils import instantiate
@@ -26,7 +27,7 @@ class TestAsyncExampleWeights:
         self,
         example_weight_config: AsyncExampleWeightConfig,
         example_weight_class: ExampleWeight,
-    ):
+    ) -> None:
         """Check that strings are correctly converted to ExampleWeight"""
         obj = instantiate(example_weight_config)
         assertEqual(obj.__class__, example_weight_class)
@@ -44,7 +45,7 @@ class TestAsyncExampleWeights:
         example_weight_config: AsyncExampleWeightConfig,
         example_weight_class: ExampleWeight,
         avg_num_examples: int,
-    ):
+    ) -> None:
         """Test that all weight computation works as expected"""
         # generate 10 random integers
         max_num_examples = 10000
