@@ -1,11 +1,12 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
 import torch
-from flsim.common.pytest_helper import assertEqual, assertTrue, assertLessEqual
+from flsim.common.pytest_helper import assertEqual, assertLessEqual, assertTrue
 from flsim.utils.data.data_utils import batchify, merge_dicts
 from flsim.utils.data.fake_data_utils import FakeDataProvider, FakeUserData
 
@@ -16,6 +17,12 @@ class TestDataUtils:
         assertEqual(list(batchify([1, 2, 3, 4, 5], 3)), [[1, 2, 3], [4, 5]])
         assertEqual(list(batchify([1, 2, 3, 4], 2)), [[1, 2], [3, 4]])
         assertEqual(list(batchify([1, 2, 3, 4], 1)), [[1], [2], [3], [4]])
+        try:
+            list(batchify([1, 2, 3, 4, 5], 8, True))
+        except AssertionError:
+            pass
+        else:
+            assert "Calling batchify on a dataset with less than batch_size with drop_last=True is not allowed"
 
     def test_merge_dicts(self) -> None:
         expected = {"a": torch.Tensor([1.0, 2.0])}
@@ -38,15 +45,15 @@ class TestDataUtils:
         expected_num_examples,
         expected_batch_size,
         expected_num_batches,
-    ):
-        assertEqual(user_dataset.num_examples(), expected_num_examples)
+    ) -> None:
+        assertEqual(user_dataset.num_train_examples(), expected_num_examples)
 
-        for i, batch in enumerate(user_dataset):
+        for i, batch in enumerate(user_dataset.train_data()):
             assertLessEqual(len(batch["data"]), expected_batch_size)
             last_batch = i
         assertEqual(last_batch + 1, expected_num_batches)
 
-    def test_fake_user_data(self):
+    def test_fake_user_data(self) -> None:
         def gen_batch(n, value=None):
             return {"data": [torch.ones(n, 10)], "label": [1] * n}
 
@@ -56,7 +63,7 @@ class TestDataUtils:
         user_dataset = FakeUserData(gen_batch, num_batches, batch_size)
         self.user_data_test_util(user_dataset, num_examples, batch_size, num_batches)
 
-    def test_fake_data_provider(self):
+    def test_fake_data_provider(self) -> None:
         def gen_batch(n, value=None):
             return {"data": [torch.ones(n, 10)], "label": [1] * n}
 
@@ -67,16 +74,20 @@ class TestDataUtils:
             gen_batch, num_batches, batch_size, num_users
         )
 
-        assertEqual(fl_data_provider.num_users(), num_users)
-        assertEqual(fl_data_provider.user_ids(), list(range(num_users)))
+        assertEqual(fl_data_provider.num_train_users(), num_users)
+        assertEqual(fl_data_provider.train_user_ids(), list(range(num_users)))
         ad_hoc_users = [0, 3, 10, 50, 99]
         num_examples = num_batches * batch_size
         for user in ad_hoc_users:
-            user_dataset = fl_data_provider.get_user_data(user)
+            user_dataset = fl_data_provider.get_train_user(user)
             self.user_data_test_util(
                 user_dataset, num_examples, batch_size, num_batches
             )
 
         self.user_data_test_util(
-            fl_data_provider.test_data(), num_examples, batch_size, num_batches
+            # pyre-fixme[16]: `Iterable` has no attribute `__getitem__`.
+            fl_data_provider.test_users()[0],
+            num_examples,
+            batch_size,
+            num_batches,
         )
