@@ -1,15 +1,12 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
 import pytest
-from flsim.common.pytest_helper import (
-    assertEqual,
-    assertRaises,
-    assertEmpty,
-)
+from flsim.common.pytest_helper import assertEmpty, assertEqual, assertRaises
 from flsim.utils.fl.common import FLModelParamUtils
 from flsim.utils.fl.personalized_model import FLModelWithPrivateModules
 from flsim.utils.tests.helpers.test_models import (
@@ -19,7 +16,7 @@ from flsim.utils.tests.helpers.test_models import (
 
 
 @pytest.fixture(scope="class")
-def prepare_fl_model_with_private_modules(request):
+def prepare_fl_model_with_private_modules(request) -> None:
     request.cls.model_with_private_modules = PersonalizedFCModel()
 
 
