@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -7,15 +8,12 @@
 import numpy as np
 import pandas as pd
 import pytest
-from flsim.common.pytest_helper import (
-    assertEqual,
-    assertAlmostEqual,
-)
+from flsim.common.pytest_helper import assertAlmostEqual, assertEqual
 from flsim.utils.fl.stats import (
+    AverageType,
     ModelSequenceNumberTracker,
     RandomVariableStatsTracker,
     RandomVariableStatsTrackerMA,
-    AverageType,
 )
 
 
@@ -97,7 +95,7 @@ class TestStatsTracker:
         assertEqual(stats_tracker.mean(), expected_mean)
         assertEqual(stats_tracker.standard_deviation(), expected_std)
 
-    def test_quantiles_tracker(self):
+    def test_quantiles_tracker(self) -> None:
         stats_tracker = RandomVariableStatsTracker(tracks_quantiles=True)
         values = []
         for i in range(100):
