@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -12,8 +13,7 @@ from dataclasses import dataclass, field
 from typing import List
 
 import torch
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
+from flsim.utils.config_utils import fullclassname, init_self_cfg
 from omegaconf import MISSING
 from torch.distributions.exponential import Exponential
 from torch.distributions.half_normal import HalfNormal
@@ -137,7 +137,6 @@ class PerUserUniformDurationDistribution(IDurationDistribution):
         pass
 
     def training_duration(self, num_training_examples: int) -> float:
-        # pyre-ignore[20]: default return shape is 1
         return self.generator.sample().item()
 
 
@@ -167,7 +166,6 @@ class PerUserExponentialDurationDistribution(IDurationDistribution):
         pass
 
     def training_duration(self, num_training_examples: int) -> float:
-        # pyre-ignore[20]: default return shape is 1
         return self.generator.sample().item()
 
 
