@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -8,9 +9,7 @@ import random
 from typing import List, Optional
 
 import numpy as np
-from flsim.utils.timing.training_duration_distribution import (
-    IDurationDistribution,
-)
+from flsim.utils.timing.training_duration_distribution import IDurationDistribution
 
 
 class TrainingTimeEstimator:
